# Checks whether the C++ compiler supports the hash_set and hash_map
# classes. If so then HAVE_HASH_CLASSES is set, and HASH_NAMESPACE
# is set to the namespace containing the classes. It also checks for
# the possible locations of the header files (top-level and ext/
# subdirectory).
dnl AC_CXX_HASH_CLASSES
AC_DEFUN([AC_CXX_HASH_CLASSES],
[AC_LANG_PUSH(C++)
AC_CHECK_HEADERS([hash_map ext/hash_map hash_set ext/hash_set])
AC_CACHE_CHECK([for hash classes], alod_cv_cxx_hash_classes,
[alod_cv_cxx_hash_classes=no
for ac_kw in std __gnu_cxx; do
  AC_COMPILE_IFELSE([AC_LANG_SOURCE(
[
#if HAVE_EXT_HASH_MAP
# include <ext/hash_map>
#endif
#if HAVE_EXT_HASH_SET
# include <ext/hash_set>
#endif
#if HAVE_HASH_MAP
# include <hash_map>
#endif
#if HAVE_HASH_SET
# include <hash_set>
#endif
$ac_kw::hash_map<int, int> hm;
$ac_kw::hash_set<int> hs;
])], [alod_cv_cxx_hash_classes=$ac_kw; break])
done
])
case $alod_cv_cxx_hash_classes in
  no) AC_DEFINE(HAVE_HASH_CLASSES,,[Define if the compiler provides hash_set and hash_map])
      AC_DEFINE(HASH_NAMESPACE,,[Define as std or __gnu_cxx if the compiler provides hash_set and hash_map]) ;;
  *)  AC_DEFINE(HAVE_HASH_CLASSES)
      AC_DEFINE_UNQUOTED(HASH_NAMESPACE, $alod_cv_cxx_hash_classes) ;;
esac
AC_LANG_POP(C++)
])

# Checks whether this is a native Win32 build viz. Mingw,
# and sets MSVC_CFLAGS to compiler options to be added.
# Currently seems to be broken
dnl AC_C_MSVC_STRUCTS
AC_DEFUN([AC_C_MSVC_STRUCTS],
[AC_CACHE_CHECK([for flags to get correct alignment], alod_cv_c_msvc_structs,
[
alod_cv_c_msvc_structs=no
if test "x$GCC" = "xyes"; then
  case `$CC --version | sed -e 's,\..*,.,' -e q` in
    2.)
      if $CC -v --help 2>/dev/null | grep fnative-struct >/dev/null; then
        alod_cv_c_msvc_structs='-fnative-struct'
      fi
      ;;
    *)
      if $CC -v --help 2>/dev/null | grep ms-bitfields >/dev/null; then
        alod_cv_c_msvc_structs='-mms-bitfields'
      fi
      ;;
  esac
fi
])
case $alod_cv_c_msvc_structs in
  no) MSVC_CFLAGS=""
      ;;
  *) MSVC_CFLAGS="$alod_cv_c_msvc_structs"
     ;;
esac
])

# Handles ImageMagick libraries. Sets MAGICK_CFLAGS and MAGICK_LIBS.
AC_DEFUN([AM_PATH_MAGICK],
[
if test -z "$MAGICK_CONFIG"; then
  AC_PATH_PROG([MAGICK_CONFIG], [Magick-config], [no])
fi
AC_CACHE_CHECK([for ImageMagick], alod_cv_path_magick,
[
  alod_cv_path_magick=no
  
  if test "$MAGICK_CONFIG" != "no"; then
    alod_cv_path_magick=yes
  fi
])
case $alod_cv_path_magick in
  yes)
    MAGICK_CFLAGS=`$MAGICK_CONFIG --cppflags`
    MAGICK_LIBS=`$MAGICK_CONFIG --ldflags --libs`
    MAGICK_LIBS=`echo $MAGICK_LIBS`   # Collapses line break
    ifelse([$1], , :, [$1])
    ;;
  no)
    MAGICK_CFLAGS=''
    MAGICK_LIBS=''
    ifelse([$2], , :, [$2])
    ;;
esac
AC_SUBST(MAGICK_CFLAGS)
AC_SUBST(MAGICK_LIBS)
])
