/*! \addtogroup gl
 * @{
 */

/*! \file
 * \brief Miscellaneous utilities for OpenGL
 */

#ifndef ALOD_ALODGL_GLMISC_H
#define ALOD_ALODGL_GLMISC_H

#include <GL/gl.h>
#include <GL/glext.h>
#include <glib.h>
#include "geom/coords.h"
#include "mesh/mesh.h"

//! Load a texture from file
GLuint alodgl_load_texture(const MeshTexture &tex, GLenum internal_format,
                           bool mipmap, GError **err);

//! Returns the matrix named by \a name
Matrix4 alodgl_get_matrix(GLenum name);

//! Multiplies the current GL matrix by \a mat
void alodgl_mult_matrix(const Matrix4 &mat);

//! Replaces the current GL matrix by \a mat
void alodgl_load_matrix(const Matrix4 &mat);

//! Wrapper around glUniform4fvARB
void alodgl_uniform_matrix(GLint location, const Matrix4 &mat);

//! Calls \c glCheckFramebufferStatusEXT and aborts on error
void alodgl_check_framebuffer_status();

#endif /* !ALOD_ALODGL_GLMISC_H */

/*! @} */
