/*! \addtogroup gl
 * @{
 */

/*! \file
 * \brief Shader management for meshrender
 */

#ifndef ALOD_ALODGL_SHADERS_H
#define ALOD_ALODGL_SHADERS_H

#include <GL/gl.h>
#include <GL/glext.h>
#include <glib.h>
#include <string>
#include <stdexcept>

//! Specifies that something went wrong loading or saving a file
class bad_shader : public std::runtime_error
{
public:
    bad_shader(const std::string &msg) : std::runtime_error(msg) {}
};

//! Sets a value to be passed by alodgl_get_shader_file_m4
void alodgl_set_shader_flag(const std::string &key, const std::string &value = "");

//! Sets a value to be passed by alodgl_get_shader_file_m4
void alodgl_set_shader_flag(const std::string &key, int value);

//! Clears all M4 defines
void alodgl_clear_shader_flags();

//! Clears a single M4 define
void alodgl_clear_shader_flag(const std::string &key);

//! Determines whether a flag is set at all
bool alodgl_has_shader_flag(const std::string &key);

//! Gets value of a flag ("" means either not present or has no value)
std::string alodgl_get_shader_flag(const std::string &key);

//! Compiles a GLSL shader, returning the handle
GLhandleARB alodgl_load_hl_shader(GLenum type, const std::string &source, const std::string &origin) throw (bad_shader);

//! Compiles a GL_ARB_vertex_program/GL_ARB_fragment_program shader, returning the handle
/*! No new ID is generated; the program overwrites any currently bound
 * program.
 */
void alodgl_load_ll_shader(GLenum target, const std::string &source, const std::string &origin) throw (bad_shader);

//! Loads shader from a file. Does not attempt to compile it.
std::string alodgl_get_shader_file(const std::string &shader_name) throw (bad_shader);

//! Loads shader from a file and runs it through M4.
std::string alodgl_get_shader_file_m4(const std::string &shader_name) throw (bad_shader);

//! Links and verifies program
void alodgl_link_program(GLhandleARB program) throw (bad_shader);

#endif /* ALOD_ALODGL_SHADERS_H */

/*! @} */
