/*! \file
 * \brief Utility functions for manipulating paths
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <glib.h>
#include <string>
#include "paths.h"

using namespace std;

string relative_path(const string &source, const string &file)
{
    gchar *dirname;
    string ans;

    if (g_path_is_absolute(file.c_str())) return file;
    dirname = g_path_get_dirname(source.c_str());
    ans = string(dirname) + G_DIR_SEPARATOR_S + file;
    g_free(dirname);
    return ans;
}

string resource_path(const string &env, const string &path, const string &file)
{
    const gchar *env_path;
    string::size_type pos = 0;

    env_path = g_getenv(env.c_str());
    if (env_path)
    {
        string env_path_s = env_path;
        while (pos != string::npos && env_path_s[pos] == G_SEARCHPATH_SEPARATOR)
            pos++;
        while (pos != string::npos)
        {
            string::size_type pos2 = env_path_s.find(G_SEARCHPATH_SEPARATOR_S, pos);
            string full = relative_path(env_path_s.substr(pos, pos2 - pos) + string(G_DIR_SEPARATOR_S), file);
            if (g_file_test(full.c_str(), G_FILE_TEST_EXISTS)) return full;
            pos = pos2;
            while (pos != string::npos && env_path_s[pos] == G_SEARCHPATH_SEPARATOR)
                pos++;
        }
    }
    return relative_path(path, file);
}
