#ifndef ALOD_COMMON_REFCOUNT_H
#define ALOD_COMMON_REFCOUNT_H

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <cstddef>
#include <memory>

template<class T> struct _refcount_base
{
    T value;
    std::size_t count;

    _refcount_base() : count(1) {}
    explicit _refcount_base(const T &t) : value(t), count(1) {}
};

template<class T, class _Alloc = std::allocator<T> > class Refcount
{
private:
    typedef _refcount_base<T> Base;
    typename _Alloc::template rebind<Base>::other alloc;
    Base *base;

    Base *get_base()
    {
        Base *b = alloc.allocate(1);
        new(b) Base();
        return b;
    }

    Base *get_base(const T &old)
    {
        Base *b = alloc.allocate(1);
        new(b) Base(old);
        return b;
    }

    void free_base(Base *b)
    {
        b->~Base();
        alloc.deallocate(b, 1);
    }

    void unique()
    {
        if (base->count > 1)
        {
            Base *old_base = base;
            base = get_base(old_base->value);
            old_base->count--;
        }
    }

public:
    Refcount() : base(get_base()) {}
    Refcount(const Refcount<T> &r) : base(r.base) { base->count++; }
    ~Refcount() { if (--base->count == 0) free_base(base); }

    Refcount<T> &operator =(const Refcount<T> &r)
    {
        Base *old_base = base;
        base = r.base;
        base->count++;
        if (--old_base->count == 0) free_base(old_base);
        return *this;
    }

    const T &get_read() const { return base->value; }
    T &get_write() { unique(); return base->value; }
};

#endif /* !ALOD_COMMON_REFCOUNT_H */
