/*! \addtogroup format
 * @{
 */

/*! \file
 * \brief Doom 3 MD5 model format support
 */

#ifndef ALOD_FORMATS_MD5_H
#define ALOD_FORMATS_MD5_H

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <iostream>
#include <string>
#include <cstddef>
#include <vector>
#include "geom/coords.h"
#include "mesh/mesh.h"
#include "formats/formats.h"

class bad_md5 : public bad_format
{
public:
    bad_md5(const std::string &msg) : bad_format(msg) {}
};

//! Loads a mesh from a Doom3 md5mesh file
void md5_load_mesh(std::istream &in, Mesh &m,
                   MeshLevel level,
                   std::vector<std::string> &comments,
                   const std::string &path,
                   const std::vector<std::string> &args) throw(bad_md5);

//! Loads purely animation information from a Doom3 md5anim file
void md5_load_anim(std::istream &in, Mesh &m, MeshLevel level,
                   std::vector<std::string> &comments,
                   const std::string &path,
                   const std::vector<std::string> &args) throw(bad_md5);


//! Saves a mesh to Doom3 MD5 format, albeit with only one joint
void md5_save_mesh(std::ostream &out, const Mesh &m,
                   const std::vector<std::string> &comments,
                   const std::string &path);

#endif /* !ALOD_FORMATS_MD5_H */

/*! @} */
