/*! \addtogroup format
 * @{
 */

/*! \file
 * \brief Interface between C and Lex for the MD5 lexer
 */

#ifndef FORMATS_MD5LEX_H
#define FORMATS_MD5LEX_H

#include <iostream>
#include <string>
#include <vector>

struct Md5Yylval
{
    int integer;
    double number;
    std::string str;
};

extern Md5Yylval md5_yylval;
extern std::istream *md5_input;
extern std::vector<std::string> *md5_comments;
extern int yylex();

/* Generic tokens */
#define MD5_TOKEN_INTEGER 300
#define MD5_TOKEN_NUMBER 301
#define MD5_TOKEN_STRING 302

/* Shared tokens */
#define MD5_TOKEN_MD5VERSION 400
#define MD5_TOKEN_NUMJOINTS 401
#define MD5_TOKEN_COMMANDLINE 402

/* md5mesh tokens */
#define MD5_TOKEN_JOINTS 500
#define MD5_TOKEN_NUMMESHES 501
#define MD5_TOKEN_MESH 502
#define MD5_TOKEN_SHADER 503
#define MD5_TOKEN_NUMVERTS 504
#define MD5_TOKEN_VERT 505
#define MD5_TOKEN_NUMTRIS 506
#define MD5_TOKEN_TRI 507
#define MD5_TOKEN_NUMWEIGHTS 508
#define MD5_TOKEN_WEIGHT 509

/* md5anim tokens */
#define MD5_TOKEN_NUMFRAMES 600
#define MD5_TOKEN_NUMANIMATEDCOMPONENTS 601
#define MD5_TOKEN_FRAMERATE 602
#define MD5_TOKEN_HIERARCHY 603
#define MD5_TOKEN_BOUNDS 604
#define MD5_TOKEN_BASEFRAME 605
#define MD5_TOKEN_FRAME 606

#endif /* !FORMATS_MD5LEX_H */

/*! @} */
