%{

/* This file handles both md5mesh and md5anim. This just leads to
 * the possibility of parse errors rather than syntax errors in
 * same cases.
 */

#include <string>
#include <iostream>
#include <cstring>
#include "formats/md5.h"
#include "formats/md5lex.h"

static std::string current_string;

#define YY_INPUT(buf, result, max_size) \
	do \
        { \
		md5_input->read(buf, max_size); \
                result = md5_input->gcount(); \
                if (!result && md5_input->eof()) result = YY_NULL; \
        } while(0)

%}

%x STRING_MODE
%x COMMENT_MODE
%x END_COMMENT_MODE

%%

MD5Version		{ return MD5_TOKEN_MD5VERSION; }
commandline		{ return MD5_TOKEN_COMMANDLINE; }
numJoints		{ return MD5_TOKEN_NUMJOINTS; }
numMeshes		{ return MD5_TOKEN_NUMMESHES; }
joints			{ return MD5_TOKEN_JOINTS; }
mesh			{ return MD5_TOKEN_MESH; }
numverts		{ return MD5_TOKEN_NUMVERTS; }
vert			{ return MD5_TOKEN_VERT; }
numtris			{ return MD5_TOKEN_NUMTRIS; }
tri			{ return MD5_TOKEN_TRI; }
numweights		{ return MD5_TOKEN_NUMWEIGHTS; }
weight			{ return MD5_TOKEN_WEIGHT; }
shader			{ return MD5_TOKEN_SHADER; }

numFrames		{ return MD5_TOKEN_NUMFRAMES; }
numAnimatedComponents	{ return MD5_TOKEN_NUMANIMATEDCOMPONENTS; }
frameRate		{ return MD5_TOKEN_FRAMERATE; }
hierarchy		{ return MD5_TOKEN_HIERARCHY; }
bounds			{ return MD5_TOKEN_BOUNDS; }
baseframe		{ return MD5_TOKEN_BASEFRAME; }
frame			{ return MD5_TOKEN_FRAME; }

\"			{ BEGIN(STRING_MODE); current_string = ""; }
<STRING_MODE>\\\n	{ /* swallow */ }
<STRING_MODE>\\.	{ current_string += yytext[1]; }
<STRING_MODE>\"		{ 
				BEGIN(INITIAL);
                                md5_yylval.str = current_string;
                                return MD5_TOKEN_STRING;
			}
<STRING_MODE>.|\n	{ current_string += yytext[0]; }
<STRING_MODE><<EOF>>	{ throw bad_md5("end of file in string"); }

[-+]?[0-9]+       		{
				md5_yylval.integer = atoi(yytext);
                                return MD5_TOKEN_INTEGER;
			}
[-+]?[0-9]+(\.[0-9]+)?([eE][-+]?[0-9]+)?	{
				md5_yylval.number = strtod(yytext, NULL);
                                return MD5_TOKEN_NUMBER;
			}

[{}()]			{ return yytext[0]; }
^"//".*\n		{
				char *front, *end;
                                front = yytext + 2;
                                end = front + strlen(front);
                                while (isspace(end[-1])) end--;
                                if (end > front) md5_comments->push_back(std::string(front, end - front));
			}
"//".*\n		{ /* ignore comments that are not at the start of a line */ }

[ \t\r\n]+		{ /* eat whitespace */ }
[a-zA-Z0-9]+   		|
.			{ throw bad_md5(std::string("unexpected character ") + yytext[0]); }

%%

int yywrap()
{
    return 1;
}
