/*! \addtogroup format
 * @{
 */

/*! \file
 * \brief Output-only support for Wavefront OBJ format
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "mesh/mesh.h"
#include "formats/obj.h"
#include "common/maputils.h"
#include <iostream>
#include <vector>
#include <string>
#include <glib.h>

using namespace std;

void obj_save_mesh(ostream &out, const Mesh &m,
                   const vector<string> &comments,
                   const string &path) throw(bad_format)
{
    const Skeleton &skeleton = m.skeleton.get_read();
    id_map<MeshPositionConstIterator, size_t>::type position_ids;
    int old_precision = out.precision(9);
    int id_pool = 0;

    for (MeshPositionConstIterator p = m.positions.real_begin(); p != m.positions.real_end(); p++)
    {
        Vector3 pos = skeleton.ssd_to_3d(p->position);
        out << "v " << pos[0] << " " << pos[1] << " " << pos[2] << "\n";
        position_ids[p] = ++id_pool;
    }
    for (MeshFaceConstIterator f = m.faces.real_begin(); f != m.faces.real_end(); f++)
    {
        out << "f";
        MeshHalfEdgeConstIterator e = f->edges();
        for (int i = 0; i < 3; i++, e = e->next())
        {
            MeshPositionConstIterator p = e->vertex()->position();
            g_assert(position_ids.count(p));
            out << " " << position_ids[p];
        }
        out << "\n";
    }

    out.precision(old_precision);
}

/*! @} */
