/*! \addtogroup geom
 * @{
 */

/*! \file
 * \brief Code to find the intersection of a number of half-planes
 */

#ifndef ALOD_GEOM_KERNEL_H
#define ALOD_GEOM_KERNEL_H

#include <vector>
#include <stdexcept>
#include "geom/coords.h"

//! Specifies that an intersection of half-planes was unbounded.
class open_kernel : public std::runtime_error
{
public:
    open_kernel() : std::runtime_error("intersection of half-planes is unbounded") {}
};

//! Determines whether \a p lies inside the kernel of the specified region
bool in_kernel(const std::vector<Vector2> &pnt,
               const std::vector<Vector2> &normal,
               const Vector2 &p) throw(open_kernel);

//! Computes the intersection of a number of half-planes.
std::vector<Vector2> kernel(const std::vector<Vector2> &pnt,
                            const std::vector<Vector2> &normal) throw(open_kernel);

#endif /* !ALOD_GEOM_KERNEL_H */

/*! @} */
