/*! \addtogroup geom
 * @{
 */

/*! \file
 * \brief A tangent plane class.
 */

#ifndef ALOD_GEOM_PLANE_H
#define ALOD_GEOM_PLANE_H

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "geom/sparse.h"
#include "geom/skeleton.h"
#include "geom/anim.h"
#include <vector>

struct TangentPlane
{
    VectorPosition tangents[2];

    TangentPlane() {}
    explicit TangentPlane(const Skeleton &skeleton)
    {
        tangents[0] = VectorPosition(skeleton);
        tangents[1] = VectorPosition(skeleton);
    }
    TangentPlane(const VectorPosition &a, const VectorPosition &b)
    {
        tangents[0] = a;
        tangents[1] = b;
    }
};

TangentPlane operator *(double s, const TangentPlane &b);

//! Takes weighted tangent planes and produces a best fit to them
TangentPlane fit_tangent_plane(const Skeleton &skeleton,
                               std::vector<TangentPlane> &planes);

#endif /* !ALOD_GEOM_PLANE_H */

/*! @} */
