/* Note: we explicitly avoid including config.h here, because ImageMagick
 * has its own version and they conflict.
 *
 * What's more, ImageMagick does a poor job of including the right headers.
 */

#include <stddef.h>
#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <magick/api.h>
#if HAVE_CONFIG_H
# undef HAVE_CONFIG_H   // avoids later inclusion
#endif
#include "common/paths.h"
#include "mesh/images.h"
#include <string>
#include <iostream>
#include <fstream>

using namespace std;

void mesh_load_texture(const string &filename,
                       const string &basename,
                       MeshTexture &tex) throw(ios::failure)
{
    ImageInfo info;
    Image *image;
    ExceptionInfo ex;

    string fname = relative_path(basename, filename);
    InitializeMagick("");
    GetExceptionInfo(&ex);
    GetImageInfo(&info);
    strncpy(info.filename, fname.c_str(), MaxTextExtent - 1);
    info.filename[MaxTextExtent - 1] = '\0';
    image = ReadImage(&info, &ex);

    if (!image) throw ios::failure("failed to load " + filename);
    try
    {
        tex.set_size(image->rows,
                     image->columns,
                     image->matte ? 4 : 3);
        PixelPacket *in = GetImagePixels(image, 0, 0, image->columns, image->rows);
        guint8 *out = tex.data();
        for (size_t i = 0; i < tex.height; i++)
        {
            for (size_t j = 0; j < tex.width; j++)
            {
                out[0] = (long) in->red * 255 / MaxRGB;
                out[1] = (long) in->green * 255 / MaxRGB;
                out[2] = (long) in->blue * 255 / MaxRGB;
                if (tex.channels == 4)
                    out[3] = 255 - ((long) in->opacity * 255 / MaxRGB);
                out += tex.channels;
                in++;
            }
        }
    }
    catch (...)
    {
        DestroyImage(image);
        throw;
    }
    DestroyImage(image);
    DestroyMagick();
}

void mesh_save_texture(const string &filename,
                       const string &basename,
                       const MeshTexture &tex) throw (ios::failure)
{
    string fname = relative_path(basename, filename);
    Image *image;
    ImageInfo *info;
    ExceptionInfo *ex;

    InitializeMagick("");
    ex = AcquireExceptionInfo();
    image = ConstituteImage(tex.width, tex.height, tex.channels == 3 ? "RGB" : "RGBA",
                            CharPixel, tex.data(), ex);
    if (ex->severity != UndefinedException)
        CatchException(ex);

    info = CloneImageInfo(NULL);
    strncpy(image->filename, fname.c_str(), MaxTextExtent - 1);
    image->filename[MaxTextExtent - 1] = '\0';
    strcpy(image->filename, fname.c_str());
    WriteImage(info, image);
    DestroyImage(image);
    DestroyImageInfo(info);
    DestroyExceptionInfo(ex);
    DestroyMagick();
}
