/*! \addtogroup mesh
 * @{
 */

/*! \file
 * \brief Texture loading and saving.
 */

#ifndef ALOD_MESH_IMAGES_H
#define ALOD_MESH_IMAGES_H

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <iostream>
#include <string>
#include "mesh/mesh.h"

//! Reads a texture from file
/*! \param fname the file to read from
 * \param basename the filename relative to which fname should be interpreted
 * \param image the output texture
 *
 * If \a filename is a relative path, it is interpreted relative to the directory
 * containing \a basename, unless \a basename is the empty string.
 *
 * If the file could not be read, a \c std::ios::failure exception is thrown.
 */
void mesh_load_texture(const std::string &fname,
                       const std::string &basename,
                       MeshTexture &image) throw(std::ios::failure);

/*! Equivalent of \c mesh_load_texture for saving textures */
void mesh_save_texture(const std::string &fname,
                       const std::string &basename,
                       const MeshTexture &image) throw (std::ios::failure);

#endif /* !ALOD_MESH_IMAGES_H */

/*! @} */
