/*! \addtogroup mesh
 * @{
 */

/*! \file
 * \brief Parametrise a Mesh into a 2D domain.
 */

#ifndef ALOD_MESH_PARAM_H
#define ALOD_MESH_PARAM_H

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "mesh/mesh.h"

//! Returns statistics about a parametrisation to callers
struct ParametrisationStatistics
{
    double texture_area;    //!< Area used by the domain itself
    double rectangle_area;  //!< Area used by the bounding rectangles for the domain
    double gutter_area;     //!< Area used by the bounding rectangles plus their gutters
};

//! Provides texture coordinates for a mesh
ParametrisationStatistics mesh_parametrise(Mesh &m, int num_patches,
                                           double limit, double compact_weight,
                                           double straighten_tolerance,
                                           double constrain_tolerance,
                                           bool no_split = false);

//! Determines the segmentation metric for the mesh, normalised for area
double mesh_segmentation_metric(const Mesh &m, double compact_weight);

//! Return Sander's stretch factor for a single triangle
double mesh_texture_triangle_stretch(MeshFaceConstIterator f, bool infinity);
//! Return Sander's stretch factor for a parametrisation
double mesh_texture_stretch(const Mesh &m, bool infinity);
//! Determines whether a parametrisation is legal
bool mesh_valid_parametrisation(const Mesh &m);

#endif /*! ALOD_MESH_PARAM_H */

/*! @} */
