#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "glee/GLee.h"
#include "render/interfaces.h"
#include "render/meshrender.h"
#include "render/bones.h"
#include "geom/skeleton.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>
#include <GL/gl.h>

using namespace std;

class RenderHelperBones : public RenderHelper
{
private:
    const Skeleton *skeleton;   //!< Skeleton passed to begin_object
public:
    RenderHelperBones(RenderHelperClassBones &parent, const RenderMesh &m);

    virtual void begin_object(const Skeleton &skeleton, double lod, double z);
    virtual void end_object();
};

RenderHelper *RenderHelperClassBones::new_instance(const RenderMesh &m)
{
    return new RenderHelperBones(*this, m);
}

void RenderHelperClassBones::begin_state(CGprogram vp, CGprogram fp, int flags)
{
    RenderHelperClass::begin_state(vp, fp, flags);
    load_simple_cg_programs(bones_vp, bones_fp);
}


RenderHelperBones::RenderHelperBones(RenderHelperClassBones &parent, const RenderMesh &m)
: RenderHelper(parent, m)
{
}

void RenderHelperBones::begin_object(const Skeleton &skeleton, double lod, double z)
{
    RenderHelper::begin_object(skeleton, lod, z);
    this->skeleton = &skeleton;
}

void RenderHelperBones::end_object()
{
    RenderHelperClassBones &klass = static_cast<RenderHelperClassBones &>(get_class());
    cgGLBindProgram(klass.bones_vp);
    cgGLBindProgram(klass.bones_fp);
    glDisable(GL_DEPTH_TEST);
    glLineWidth(3.0);
    glColor4f(0.0f, 1.0f, 0.0f, 1.0f);

    glBegin(GL_LINES);
    size_t b = 0;
    for (Skeleton::const_iterator i = skeleton->begin(); i != skeleton->end(); i++, b++)
    {
        Matrix4 abs = skeleton->get_absolute(b);
        Vector3 head = matrix4_to_translate3(abs);
        Vector3 tail = head + matrix4_to_matrix3(abs) * i->tail;
        glVertex3d(head[0], head[1], head[2]);
        glVertex3d(tail[0], tail[1], tail[2]);
    }
    glEnd();

    glLineWidth(1.0);
    glEnable(GL_DEPTH_TEST);
    cgGLBindProgram(klass.get_vertex_program());
    cgGLBindProgram(klass.get_fragment_program());
    RenderHelper::end_object();
}
