#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "glee/GLee.h"
#include "render/interfaces.h"
#include "render/meshrender.h"
#include "render/bonespheres.h"
#include "geom/bounding.h"
#include "geom/skeleton.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>
#include <GL/gl.h>

using namespace std;

class RenderHelperBoneSpheres : public RenderHelper
{
private:
    const Skeleton *skeleton;   //!< Skeleton passed to \c begin_object
public:
    RenderHelperBoneSpheres(RenderHelperClassBoneSpheres &parent, const RenderMesh &m);

    virtual void begin_object(const Skeleton &skeleton, double lod, double z);
    virtual void end_object();
};


RenderHelper *RenderHelperClassBoneSpheres::new_instance(const RenderMesh &m)
{
    return new RenderHelperBoneSpheres(*this, m);
}

void RenderHelperClassBoneSpheres::begin_state(CGprogram vp, CGprogram fp, int flags)
{
    RenderHelperClass::begin_state(vp, fp, flags);

    load_simple_cg_programs(sphere_vp, sphere_fp);
}


RenderHelperBoneSpheres::RenderHelperBoneSpheres(RenderHelperClassBoneSpheres &parent, const RenderMesh &m)
: RenderHelper(parent, m)
{
}

void RenderHelperBoneSpheres::begin_object(const Skeleton &skeleton, double lod, double z)
{
    RenderHelper::begin_object(skeleton, lod, z);
    this->skeleton = &skeleton;
}

void RenderHelperBoneSpheres::end_object()
{
    RenderHelperClassBoneSpheres &klass = static_cast<RenderHelperClassBoneSpheres &>(get_class());

    check_cg_error();
    cgGLBindProgram(klass.sphere_vp);
    cgGLBindProgram(klass.sphere_fp);
    glDepthMask(GL_FALSE);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glColor4f(1.0f, 0.0f, 0.0f, 0.3f);

    for (size_t i = 1; i < skeleton->size(); i++)
    {
        const SkeletonBone &b = skeleton->get_bone(i);
        BoundingSphere s;
        const Matrix4 &g = skeleton->get_absolute(b.parent);

        s = g * b.translate_bounds;
        glPushMatrix();
        glTranslated(s.centre[0], s.centre[1], s.centre[2]);
        glScaled(s.radius, s.radius, s.radius);
        glCallList(sphere_list());
        glPopMatrix();
    }

    glBlendFunc(GL_ONE, GL_ZERO);
    glDisable(GL_BLEND);
    glDepthMask(GL_TRUE);
    check_cg_error();
    cgGLBindProgram(klass.get_vertex_program());
    cgGLBindProgram(klass.get_fragment_program());
    check_cg_error();

    RenderHelper::end_object();
}
