#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "glee/GLee.h"
#include "render/interfaces.h"
#include "render/meshrender.h"
#include "render/corners.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>
#include <GL/gl.h>

using namespace std;

class RenderHelperCorners : public RenderHelper
{
private:
    vector<GLuint> corner_buffers;
    vector<GLsizei> num_corners;
    int ilod;                      //!< Level given to begin_object

public:
    RenderHelperCorners(RenderHelperClassCorners &parent, const RenderMesh &m);
    ~RenderHelperCorners();

    virtual void begin_object(const Skeleton &skeleton, double lod, double z);
    virtual void end_object();
};


RenderHelper *RenderHelperClassCorners::new_instance(const RenderMesh &m)
{
    return new RenderHelperCorners(*this, m);
}

void RenderHelperClassCorners::begin_state(CGprogram vp, CGprogram fp, int flags)
{
    RenderHelperClass::begin_state(vp, fp, flags);

    load_simple_cg_programs(corner_vp, corner_fp);
    glPolygonOffset(0.6, 4.0);
    glEnable(GL_POLYGON_OFFSET_FILL);
}

void RenderHelperClassCorners::end_state()
{
    glPolygonOffset(0.0, 0.0);
    glDisable(GL_POLYGON_OFFSET_FILL);

    RenderHelperClass::end_state();
}


RenderHelperCorners::RenderHelperCorners(RenderHelperClassCorners &parent, const RenderMesh &m)
: RenderHelper(parent, m)
{
    vector<GLuint> corners;
    for (size_t i = 0; i < m.levels.size(); i++)
    {
        GLuint buffer;
        const RenderMeshLevel &l = m.levels[i];
        corners.clear();
        for (GLuint v = 0; v < l.vertices.size(); v++)
        {
            if (l.vertices[v].first->position()->patch_corner)
                corners.push_back(v);
        }
        num_corners.push_back(corners.size());

        glGenBuffers(1, &buffer);
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, buffer);
        glBufferData(GL_ELEMENT_ARRAY_BUFFER, corners.size() * sizeof(GLuint),
                     &corners[0], GL_STATIC_DRAW);
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
        corner_buffers.push_back(buffer);
    }
}

RenderHelperCorners::~RenderHelperCorners()
{
    for (size_t i = 0; i < corner_buffers.size(); i++)
        glDeleteBuffers(1, &corner_buffers[i]);
}

void RenderHelperCorners::begin_object(const Skeleton &skeleton, double lod, double z)
{
    RenderHelper::begin_object(skeleton, lod, z);
    ilod = rmesh.int_lod(lod);
}

void RenderHelperCorners::end_object()
{
    GLint max_attribs;
    RenderHelperClassCorners &klass = static_cast<RenderHelperClassCorners &>(get_class());

    cgGLBindProgram(klass.corner_vp);
    cgGLBindProgram(klass.corner_fp);
    check_cg_error();
    glEnable(GL_POINT_SMOOTH);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glPointSize(16.0f);
    glColor3f(0.0f, 0.0f, 1.0f);
    /* Ensure that colour is not taken from an active array */
    glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT);
    glGetIntegerv(GL_MAX_VERTEX_ATTRIBS_ARB, &max_attribs);
    for (int i = 1; i < max_attribs; i++)
        glDisableVertexAttribArrayARB(i);
    glDisableClientState(GL_COLOR_ARRAY);

    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, corner_buffers[ilod]);
    glDrawRangeElements(GL_POINTS, 0, rmesh.levels[ilod].vertices.size() - 1,
                        num_corners[ilod], GL_UNSIGNED_INT, NULL);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

    glPopClientAttrib();
    glPointSize(1.0f);
    glBlendFunc(GL_ONE, GL_ZERO);
    glDisable(GL_BLEND);
    glDisable(GL_POINT_SMOOTH);
    cgGLBindProgram(klass.get_vertex_program());
    cgGLBindProgram(klass.get_fragment_program());
    check_cg_error();

    RenderHelper::end_object();
}
