#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "glee/GLee.h"
#include <GL/gl.h>
#include "render/interfaces.h"
#include "render/meshrender.h"
#include "render/decal.h"
#include "alodgl/glmisc.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>

class RenderHelperDecal : public RenderHelper
{
private:
    GLuint decal_texture;

public:
    RenderHelperDecal(RenderHelperClassDecal &parent, const RenderMesh &m);
    ~RenderHelperDecal();

    virtual void begin_object(const Skeleton &skeleton, double lod, double z);
};


RenderHelper *RenderHelperClassDecal::new_instance(const RenderMesh &m)
{
    return new RenderHelperDecal(*this, m);
}

void RenderHelperClassDecal::set_parameters(CGparameter vparameter, CGparameter fparameter)
{
    decal_parameter = cgGetNamedStructParameter(fparameter, "texture");
    check_cg_error();
    g_assert(decal_parameter);
}

RenderHelperDecal::RenderHelperDecal(RenderHelperClassDecal &parent, const RenderMesh &m)
: RenderHelper(parent, m), decal_texture(0)
{
    if (m.mesh.textures.empty())
    {
        GLubyte white = 255;
        GLint old;
        glGetIntegerv(GL_TEXTURE_BINDING_2D, &old);
        glGenTextures(1, &decal_texture);
        glBindTexture(GL_TEXTURE_2D, decal_texture);
        glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE8, 1, 1, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, &white);
        glBindTexture(GL_TEXTURE_2D, old);
    }
    else
    {
        GError *error = NULL;
        decal_texture = alodgl_load_texture(m.mesh.textures.front(), GL_COMPRESSED_RGB, true, &error);
        if (!decal_texture)
            g_error(error->message);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
    }
}

RenderHelperDecal::~RenderHelperDecal()
{
    glDeleteTextures(1, &decal_texture);
}

void RenderHelperDecal::begin_object(const Skeleton &skeleton, double lod, double z)
{
    RenderHelperClassDecal &klass = static_cast<RenderHelperClassDecal &>(get_class());

    RenderHelper::begin_object(skeleton, lod, z);
    cgGLSetTextureParameter(klass.decal_parameter, decal_texture);
    cgGLEnableTextureParameter(klass.decal_parameter);
}
