#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <glee/GLee.h>
#include "render/interfaces.h"
#include "render/meshrender.h"
#include "render/geometry_sse.h"
#include "render/distance.h"
#include "alodgl/glmisc.h"
#include <glib.h>
#include <GL/gl.h>
#include <Cg/cg.h>
#include <Cg/cgGL.h>
#include <vector>
#include <string>

using namespace std;

class RenderHelperDistance : public RenderHelper
{
private:
    RenderHelper *geometry_instance;
public:
    RenderHelperDistance(RenderHelperClassDistance &parent, const RenderMesh &m);
    virtual ~RenderHelperDistance();

    virtual void begin_object(const Skeleton &skeleton, double lod, double z);
};


RenderHelper *RenderHelperClassDistance::new_instance(const RenderMesh &m)
{
    return new RenderHelperDistance(*this, m);
}

CGparameter RenderHelperClassDistance::begin_fragment_program(CGprogram fp)
{
    /*** Generate the textures ***/
    CGparameter geometry_cg_interface, scale_parameter;

    const char *args[] = {"-DTEXCOORDS", NULL};
    int l, s;

    generate_vp = load_cg_program("rtt_vp.cg", global_cg_vertex_profile(), args);
    generate_fp = load_cg_program("distance_fp.cg", global_cg_fragment_profile(), args);
    geometry_class = new RenderHelperClassGeometrySSE(true);
    geometry_cg_class = geometry_class->begin_vertex_program(generate_vp);
    geometry_cg_interface = cgGetNamedParameter(generate_vp, "geometry");
    cgConnectParameter(geometry_cg_class, geometry_cg_interface);
    check_cg_error();

    cgCompileProgram(generate_vp); check_cg_error();
    cgCompileProgram(generate_fp); check_cg_error();
    cgGLLoadProgram(generate_vp);  check_cg_error();
    cgGLLoadProgram(generate_fp);  check_cg_error();
    geometry_class->begin_state(generate_vp, generate_fp, RENDER_FLAG_TEXCOORDS);
    geometry_class->end_state();

    GLint old;
    glGetIntegerv(GL_TEXTURE_BINDING_2D, &old);
    glGenFramebuffersEXT(2, fbos);
    glGenTextures(2, textures);
    for (int i = 0; i < 2; i++)
    {
        glBindTexture(GL_TEXTURE_2D, textures[i]);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
        l = 0;
        s = size;
        while (s)
        {
            glTexImage2D(GL_TEXTURE_2D, l, GL_RGBA16F_ARB, s, s, 0,
                         GL_RGBA, GL_FLOAT, NULL);
            l++;
            s >>= 1;
        }

        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fbos[i]);
        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
                                  GL_TEXTURE_2D, textures[i], 0);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);

        check_cg_error();
    }
    glBindTexture(GL_TEXTURE_2D, old);

    /*** Set the scale ***/

    CGparameter cg_class = RenderHelperClass::begin_fragment_program(fp);
    scale_parameter = cgGetNamedStructParameter(cg_class, "scale");
    cgSetParameter1f(scale_parameter, scale);
    return cg_class;
}

void RenderHelperClassDistance::set_parameters(CGparameter vparameter, CGparameter fparameter)
{
    position_parameters[0] = cgGetNamedStructParameter(fparameter, "position_textures[0]");
    position_parameters[1] = cgGetNamedStructParameter(fparameter, "position_textures[1]");
}

void RenderHelperClassDistance::end_state()
{
    glDeleteFramebuffersEXT(2, fbos);
    glDeleteTextures(2, textures);
    cgDestroyProgram(generate_vp);
    cgDestroyProgram(generate_fp);
    cgDestroyParameter(geometry_cg_class);
    delete geometry_class;
}


RenderHelperDistance::RenderHelperDistance(RenderHelperClassDistance &parent, const RenderMesh &m)
: RenderHelper(parent, m)
{
    geometry_instance = parent.geometry_class->new_instance(m);
}

RenderHelperDistance::~RenderHelperDistance()
{
    delete geometry_instance;
}

void RenderHelperDistance::begin_object(const Skeleton &skeleton, double lod, double z)
{
    GLint old_viewport[4];
    GLint attribs;
    RenderHelperClassDistance &klass = static_cast<RenderHelperClassDistance &>(get_class());

    RenderHelper::begin_object(skeleton, lod, z);

    glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT);
    glDisable(GL_DEPTH_TEST);
    glGetIntegerv(GL_VIEWPORT, old_viewport);
    glViewport(0, 0, klass.size, klass.size);
    glGetIntegerv(GL_MAX_VERTEX_ATTRIBS, &attribs);
    for (int i = 0; i < attribs; i++)
        glDisableVertexAttribArray(i);
    klass.geometry_class->begin_state(klass.generate_vp, klass.generate_fp,
                                      RENDER_FLAG_TEXCOORDS);
    cgGLBindProgram(klass.generate_vp);
    cgGLBindProgram(klass.generate_fp);

    for (int i = 0; i < 2; i++)
    {
        GLint old;
        double l = i ? lod : 0.0;
        int il = rmesh.int_lod(l);

        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, klass.fbos[i]);
        alodgl_check_framebuffer_status();
        geometry_instance->begin_object(skeleton, l, z);
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, rmesh.levels[il].index_buffer);
        glClear(GL_COLOR_BUFFER_BIT);
        glDrawRangeElements(GL_TRIANGLES, 0, rmesh.levels[il].vertices.size() - 1,
                            3 * rmesh.levels[il].faces.size(), GL_UNSIGNED_INT, NULL);
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
        geometry_instance->end_object();

        glGetIntegerv(GL_TEXTURE_BINDING_2D, &old);
        glBindTexture(GL_TEXTURE_2D, klass.textures[i]);
        glGenerateMipmapEXT(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, old);
    }

    klass.geometry_class->end_state();
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
    glViewport(old_viewport[0], old_viewport[1], old_viewport[2], old_viewport[3]);
    glEnable(GL_DEPTH_TEST);
    glPopClientAttrib();
    cgGLBindProgram(get_class().get_vertex_program());
    cgGLBindProgram(get_class().get_fragment_program());
    cgGLSetTextureParameter(klass.position_parameters[0], klass.textures[0]);
    cgGLSetTextureParameter(klass.position_parameters[1], klass.textures[1]);
    cgGLEnableTextureParameter(klass.position_parameters[0]);
    cgGLEnableTextureParameter(klass.position_parameters[1]);
}
