#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "render/geometry_sse.h"
#include "render/geometry_col.h"
#include "render/geometry_frag.h"
#include "render/geometry_texture.h"
#include "render/normal_vertex.h"
#include "render/normal_static.h"
#include "render/normal_dynamic.h"
#include "render/normal_null.h"
#include "render/lighting_dot.h"
#include "render/lighting_cube.h"
#include "render/lighting_normal.h"
#include "render/isolines.h"
#include "render/decal.h"
#include "render/patches.h"
#include "render/influences.h"
#include "render/distance.h"
#include "render/spheres.h"
#include "render/bonespheres.h"
#include "render/bones.h"
#include "render/corners.h"
#include "render/factory.h"

template<class HelperClass> static RenderHelperClass *instantiate_basic()
{
    return new HelperClass();
}

template<class HelperClass, int scale, bool checkerboard> static RenderHelperClass *instantiate_iso()
{
    return new HelperClass((float) scale, checkerboard);
}

template<class HelperClass, int size, int scale> static RenderHelperClass *instantiate_distance()
{
    return new HelperClass(size, (float) scale);
}

template<class HelperClass, int tile_size> static RenderHelperClass *instantiate_int()
{
    return new HelperClass(tile_size);
}

template<class HelperClass, bool v> static RenderHelperClass *instantiate_bool()
{
    return new HelperClass(v);
}

extern const FactoryItem factory[] =
{
    { "SSE by vertex (buffered)", "<control>R", RENDER_HELPER_GEOMETRY, instantiate_bool<RenderHelperClassGeometrySSE, true> },
    { "SSE by vertex (streamed)", "<control>S", RENDER_HELPER_GEOMETRY, instantiate_bool<RenderHelperClassGeometrySSE, false> },
    { "SSE by bone",     "<control>C", RENDER_HELPER_GEOMETRY, instantiate_basic<RenderHelperClassGeometryCol> },
    { "FP with loop",    "<control>F", RENDER_HELPER_GEOMETRY, instantiate_basic<RenderHelperClassGeometryFrag> },
    { "Vertex textures", "<control>T", RENDER_HELPER_GEOMETRY, instantiate_basic<RenderHelperClassGeometryTexture> },

    { "Vertex",          NULL,         RENDER_HELPER_NORMAL,   instantiate_basic<RenderHelperClassNormalVertex> },
    { "Static pixel",    NULL,         RENDER_HELPER_NORMAL,   instantiate_basic<RenderHelperClassNormalStatic> },
    { "Dynamic pixel (2x2)", NULL,     RENDER_HELPER_NORMAL,   instantiate_int<RenderHelperClassNormalDynamic, 2> },
    { "Dynamic pixel (4x4)", NULL,     RENDER_HELPER_NORMAL,   instantiate_int<RenderHelperClassNormalDynamic, 4> },
    { "Dynamic pixel (8x8)", NULL,     RENDER_HELPER_NORMAL,   instantiate_int<RenderHelperClassNormalDynamic, 8> },
    { "Dynamic pixel (16x16)", NULL,   RENDER_HELPER_NORMAL,   instantiate_int<RenderHelperClassNormalDynamic, 16> },
    { "Dynamic pixel (32x32)", NULL,   RENDER_HELPER_NORMAL,   instantiate_int<RenderHelperClassNormalDynamic, 32> },
    { "Null",            NULL,         RENDER_HELPER_NORMAL,   instantiate_basic<RenderHelperClassNormalNull> },

    { "Dot product",     NULL,         RENDER_HELPER_LIGHTING, instantiate_basic<RenderHelperClassLightingDot> },
    { "Cube map",        NULL,         RENDER_HELPER_LIGHTING, instantiate_int<RenderHelperClassLightingCube, 32> },
    { "Normal",          NULL,         RENDER_HELPER_LIGHTING, instantiate_basic<RenderHelperClassLightingNormal> },

    { "Texture",         NULL,         RENDER_HELPER_SHADER,   instantiate_basic<RenderHelperClassDecal> },
    { "Patches",         NULL,         RENDER_HELPER_SHADER,   instantiate_bool<RenderHelperClassPatches, false> },
    { "Patches (rand)",  NULL,         RENDER_HELPER_SHADER,   instantiate_bool<RenderHelperClassPatches, true> },
    { "Influences",      NULL,         RENDER_HELPER_SHADER,   instantiate_basic<RenderHelperClassInfluences> },
    { "Error (coarse)",  NULL,         RENDER_HELPER_SHADER,   instantiate_distance<RenderHelperClassDistance, 512, 20> },
    { "Error (fine)",    NULL,         RENDER_HELPER_SHADER,   instantiate_distance<RenderHelperClassDistance, 512, 100> },
    { "Isolines",        NULL,         RENDER_HELPER_SHADER,   instantiate_iso<RenderHelperClassIsolines, 64, false> },
    { "Isolines (fine)", NULL,         RENDER_HELPER_SHADER,   instantiate_iso<RenderHelperClassIsolines, 1024, false> },
    { "Checkerboard",    NULL,         RENDER_HELPER_SHADER,   instantiate_iso<RenderHelperClassIsolines, 64, true> },

    { "Bounding spheres", NULL,        RENDER_HELPER_OTHER,    instantiate_basic<RenderHelperClassSpheres> },
    { "Bone spheres",    NULL,         RENDER_HELPER_OTHER,    instantiate_basic<RenderHelperClassBoneSpheres> },
    { "Bones",           NULL,         RENDER_HELPER_OTHER,    instantiate_basic<RenderHelperClassBones> },
    { "Corners",         NULL,         RENDER_HELPER_OTHER,    instantiate_basic<RenderHelperClassCorners> },

    { NULL,              NULL,         RENDER_HELPER_GEOMETRY, NULL }
};
