#ifndef ALOD_RENDER_GEOMETRY_RTVB
#define ALOD_RENDER_GEOMETRY_RTVB

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "render/interfaces.h"
#include "render/meshrender.h"
#include "render/geometry.h"
#include "render/transform_texture.h"
#include "geom/skeleton.h"
#include <GL/gl.h>
#include <Cg/cg.h>
#include <string>
#include <utility>

//! Generic helper for all render-to-vertex-buffer (RTVB) helpers
class RenderHelperClassGeometryRTVB : public RenderHelperClassGeometry
{
private:
    CGprogram rtvb_vp, rtvb_fp;
    GLuint fbo, rbo[3];
    GLint rbo_width, rbo_height;
    TransformTexture transform_texture;

    friend class RenderHelperGeometryRTVB;

    void set_min_rbo_size(int width, int height);
protected:
    virtual GLenum buffer_usage() { return GL_STREAM_COPY; }

    virtual std::string rtvb_vertex_program_file() const = 0;
    virtual std::string rtvb_fragment_program_file() const = 0;

    CGprogram rtvb_vertex_program() { return rtvb_vp; }
    CGprogram rtvb_fragment_program() { return rtvb_fp; }
public:
    RenderHelperClassGeometryRTVB() : RenderHelperClassGeometry(false) {}
    virtual void begin_state(CGprogram vp, CGprogram fp, int flags);
    virtual void end_state();
};


class RenderHelperGeometryRTVB : public RenderHelperGeometry
{
public:
    RenderHelperGeometryRTVB(RenderHelperClassGeometryRTVB &parent, const RenderMesh &m);
    virtual ~RenderHelperGeometryRTVB();

    RenderHelperClassGeometryRTVB &get_class_geometry_rtvb()
    { return static_cast<RenderHelperClassGeometryRTVB &>(get_class()); }

    //! Sets up appropriate FBO state for render-to-vertex-buffer
    virtual void begin_object(const Skeleton &skeleton, double lod, double z);
    //! Balances the effects of \c begin_object and extracts the vertex data
    void restore_object(const Skeleton &skeleton, double lod, double z);
private:
    struct RTVBLevel
    {
        GLint rbo_width, rbo_height;
    };
    std::vector<RTVBLevel> rtvb_levels;

protected:
    CGprogram rtvb_vertex_program() { return get_class_geometry_rtvb().rtvb_vp; }
    CGprogram rtvb_fragment_program() { return get_class_geometry_rtvb().rtvb_fp; }
    GLuint get_rbo(int r) { return get_class_geometry_rtvb().rbo[r]; }

    void get_rbo_size(int level, GLint &rbo_width, GLint &rbo_height) const;
    void set_rbo_size(int level, GLint rbo_width, GLint rbo_height);
    //! Returns the built transform texture
    GLuint update_transform_texture(const Skeleton &skeleton);
};

#endif /* !ALOD_RENDER_GEOMETRY_RTVB */
