#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "glee/GLee.h"
#include <GL/gl.h>
#include "render/interfaces.h"
#include "render/meshrender.h"
#include "render/rendermesh.h"
#include "render/influences.h"
#include <vector>
#include <string>
#include <glib.h>
#include <Cg/cg.h>
#include <Cg/cgGL.h>

using namespace std;

class RenderHelperInfluences : public RenderHelper
{
private:
    vector<GLuint> influence_buffers;
public:
    RenderHelperInfluences(RenderHelperClassInfluences &parent, const RenderMesh &m);

    virtual void begin_object(const Skeleton &skeleton, double lod, double z);
};


RenderHelper *RenderHelperClassInfluences::new_instance(const RenderMesh &m)
{
    return new RenderHelperInfluences(*this, m);
}

void RenderHelperClassInfluences::begin_state(CGprogram vp, CGprogram fp, int flags)
{
    RenderHelperClass::begin_state(vp, fp, flags);

    influences_parameter = cgGetNamedParameter(vp, "influence_count");
    check_cg_error();
    cgGLEnableClientState(influences_parameter);
}

void RenderHelperClassInfluences::end_state()
{
    cgGLDisableClientState(influences_parameter);
}


RenderHelperInfluences::RenderHelperInfluences(RenderHelperClassInfluences &parent, const RenderMesh &m)
: RenderHelper(parent, m)
{
    influence_buffers.resize(m.levels.size(), 0);
    for (size_t i = 0; i < m.levels.size(); i++)
    {
        glGenBuffers(1, &influence_buffers[i]);
        glBindBuffer(GL_ARRAY_BUFFER, influence_buffers[i]);
        glBufferData(GL_ARRAY_BUFFER, sizeof(GLfloat) * m.levels[i].vertices.size(),
                     NULL, GL_STATIC_DRAW);
        GLfloat *ptr = (GLfloat *) glMapBuffer(GL_ARRAY_BUFFER, GL_WRITE_ONLY);

        for (size_t j = 0; j < m.levels[i].vertices.size(); j++)
        {
            const VectorPosition &pos = m.levels[i].vertices[j].first->position()->position;
            *ptr++ = distance(pos.begin(), pos.end());
        }
        GLboolean success = glUnmapBuffer(GL_ARRAY_BUFFER);
        if (!success) g_error("glUnmapBuffer failed");
    }
}

void RenderHelperInfluences::begin_object(const Skeleton &skeleton, double lod, double z)
{
    RenderHelperClassInfluences &klass = static_cast<RenderHelperClassInfluences &>(get_class());

    RenderHelper::begin_object(skeleton, lod, z);
    int ilod = rmesh.int_lod(lod);
    glBindBuffer(GL_ARRAY_BUFFER, influence_buffers[ilod]);
    cgGLSetParameterPointer(klass.influences_parameter, 1, GL_FLOAT, 0, NULL);
}
