#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "glee/GLee.h"
#include <GL/gl.h>
#include "render/interfaces.h"
#include "render/isolines.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>

RenderHelper *RenderHelperClassIsolines::new_instance(const RenderMesh &m)
{
    return new RenderHelper(*this, m);
}

CGparameter RenderHelperClassIsolines::begin_fragment_program(CGprogram fragment_program)
{
    /* We do this here rather than as part of begin_state, because the latter
     * makes it difficult to call glBindTexture without clobbering the
     * bind-to-render settings.
     */
    GLubyte data[64][64];
    GLint old;

    glGetIntegerv(GL_TEXTURE_BINDING_2D, &old);
    glGenTextures(1, &texture);
    glBindTexture(GL_TEXTURE_2D, texture);
    if (checkerboard)
    {
        for (int i = 0; i < 64; i++)
            for (int j = 0; j < 64; j++)
                data[i][j] = ((i < 32) ^ (j < 32)) ? 255 : 127;
    }
    else
    {
        memset(data, 255, sizeof(data));
        for (int i = 0; i < 64; i++)
            data[i][0] = data[0][i] = data[i][63] = data[63][i] = 0;
    }
    glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE4, 64, 64, 0, GL_LUMINANCE,
                 GL_UNSIGNED_BYTE, data);
    glBindTexture(GL_TEXTURE_2D, old);

    CGparameter cg_class = RenderHelperClass::begin_fragment_program(fragment_program);
    CGparameter scale_parameter = cgGetNamedStructParameter(cg_class, "scale");
    g_assert(scale_parameter);
    cgSetParameterValuefr(scale_parameter, 1, &scale);

    return cg_class;
}

void RenderHelperClassIsolines::set_parameters(CGparameter vparameter, CGparameter fparameter)
{
    CGparameter texture_parameter = cgGetNamedStructParameter(fparameter, "texture");
    g_assert(texture_parameter);
    cgGLSetTextureParameter(texture_parameter, texture);
    cgGLEnableTextureParameter(texture_parameter);
}

void RenderHelperClassIsolines::begin_state(CGprogram vp, CGprogram fp, int flags)
{
    RenderHelperClass::begin_state(vp, fp, flags);
}

void RenderHelperClassIsolines::end_state()
{
    glDeleteTextures(1, &texture);
    RenderHelperClass::end_state();
}
