#ifndef ALOD_RENDER_NORMAL_DYNAMIC_H
#define ALOD_RENDER_NORMAL_DYNAMIC_H

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "render/interfaces.h"
#include "render/meshrender.h"
#include "render/transform_texture.h"
#include <Cg/cg.h>
#include <GL/gl.h>
#include <vector>
#include <string>

class RenderHelperClassNormalDynamic : public RenderHelperClass
{
private:
    struct NormalTexture
    {
        GLuint texture_id;
        GLuint fbo_id;

        NormalTexture() : texture_id(0), fbo_id(0) {}
        void initialise(int size);
    };

    int tile_size;

    CGprogram generate_vp, generate_fp;
    CGparameter bone_parameter, tangent_parameters[2], transform_parameter; // for generation
    CGparameter normal_parameter;                                           // for rendering

    std::vector<NormalTexture> normal_textures; // one per power of two
    TransformTexture transform_texture;

    //! Binds an appropriate FBO for the given texture level
    /*! The return value is the texture ID to bind after rendering to it
     */
    GLuint bind_fbo(int log_size);

    friend class RenderHelperNormalDynamic;

    //! Prevent default instantiation
    RenderHelperClassNormalDynamic() {}
protected:
    virtual std::string fragment_program_file() const { return "normal_dynamic.cg"; }
    virtual std::string fragment_program_class() const { return "NormalDynamicFP"; }
public:
    explicit RenderHelperClassNormalDynamic(int tile_size);

    virtual RenderHelperType type() const { return RENDER_HELPER_NORMAL; }
    virtual int flags() const { return RENDER_FLAG_TEXCOORDS; }
    virtual RenderHelper *new_instance(const RenderMesh &m);

    virtual void set_parameters(CGparameter vparameter, CGparameter fparameter);
    virtual void end_state();
};

#endif /* !ALOD_RENDER_NORMAL_DYNAMIC_H */
