#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "glee/GLee.h"
#include "render/interfaces.h"
#include "render/meshrender.h"
#include "render/spheres.h"
#include "geom/bounding.h"
#include "geom/skeleton.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>
#include <GL/gl.h>

using namespace std;

class RenderHelperSpheres : public RenderHelper
{
public:
    RenderHelperSpheres(RenderHelperClassSpheres &parent, const RenderMesh &m);

    virtual void end_object();
};


RenderHelper *RenderHelperClassSpheres::new_instance(const RenderMesh &m)
{
    return new RenderHelperSpheres(*this, m);
}

void RenderHelperClassSpheres::begin_state(CGprogram vp, CGprogram fp, int flags)
{
    RenderHelperClass::begin_state(vp, fp, flags);

    load_simple_cg_programs(sphere_vp, sphere_fp);
}


RenderHelperSpheres::RenderHelperSpheres(RenderHelperClassSpheres &parent, const RenderMesh &m)
: RenderHelper(parent, m)
{
}

void RenderHelperSpheres::end_object()
{
    RenderHelperClassSpheres &klass = static_cast<RenderHelperClassSpheres &>(get_class());

    check_cg_error();
    cgGLBindProgram(klass.sphere_vp);
    cgGLBindProgram(klass.sphere_fp);
    glDepthMask(GL_FALSE);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glColor4f(0.0f, 0.0f, 1.0f, 0.3f);
    glPushMatrix();
    glTranslated(rmesh.bounds.centre[0], rmesh.bounds.centre[1], rmesh.bounds.centre[2]);
    glScaled(rmesh.bounds.radius, rmesh.bounds.radius, rmesh.bounds.radius);

    glCallList(sphere_list());

    glPopMatrix();
    glBlendFunc(GL_ONE, GL_ZERO);
    glDisable(GL_BLEND);
    glDepthMask(GL_TRUE);
    check_cg_error();
    cgGLBindProgram(klass.get_vertex_program());
    cgGLBindProgram(klass.get_fragment_program());
    check_cg_error();

    RenderHelper::end_object();
}
