#if HAVE_CONFIG_H
# include <config.h>
#endif
#include "glee/GLee.h"
#include "transform_texture.h"
#include <GL/gl.h>

TransformTexture::~TransformTexture()
{
    end();
}

GLuint TransformTexture::update(const Skeleton &skeleton)
{
    if (!transform_texture)
    {
        glGenTextures(1, &transform_texture);
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB, transform_texture);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    }
    else
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB, transform_texture);

    int bones = skeleton.size();
    GLfloat tex_data[12 * bones];
    int index = 0;
    for (int i = 0; i < bones; i++)
    {
        Matrix4 m = skeleton.get_absolute(i);
        for (int j = 0; j < 3; j++)
            for (int k = 0; k < 4; k++)
                tex_data[index++] = m[j][k];
    }
    if (bones > transform_texture_height)
    {
        glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_RGBA16F_ARB, 3, bones, 0,
                     GL_RGBA, GL_FLOAT, tex_data);
        transform_texture_height = bones;
    }
    else
        glTexSubImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, 0, 0, 3, bones,
                        GL_RGBA, GL_FLOAT, tex_data);
    return transform_texture;
}

void TransformTexture::end()
{
    glDeleteTextures(1, &transform_texture);
    transform_texture = 0;
    transform_texture_height = 0;
}
