import string, struct, os.path, sys
from Blender import *
from Blender.Mathutils import *
from armature_matrix import *

def do_work(obj):
        # Dirty hack: update the Scene inside here, to guarantee that armatures
        # are in the current pose. To prevent infinite recursion, we set a
        # layer flag in the scene to indicate recursion. This is also
        # why we use a function: it lets us use return;
        scene = Blender.Scene.GetCurrent()
        if scene.Layers & 0x80000: return
        scene.Layers |= 0x80000
        scene.update(1)
        scene.Layers &= ~0x80000

        class Properties:
                def __init__(self, obj):
                        self.filename = obj.getProperty('alm_filename').getData()
                        self.bones_offset = int(obj.getProperty('alm_bones_offset').getData())
                        self.positions_offset = int(obj.getProperty('alm_positions_offset').getData())
                        self.frames_offset = int(obj.getProperty('alm_frames_offset').getData())
                        self.num_bones = int(obj.getProperty('alm_bones').getData())
                        self.num_frames = int(obj.getProperty('alm_frames').getData())
                        self.animate_model = int(obj.getProperty('alm_animate_model').getData())

        class AlmVectorElement:
                def __init__(self):
                        self.bone = -1
                        self.values = Vector([0.0, 0.0, 0.0, 0.0])

        class AlmVector:
                def __init__(self):
                        self.elements = []

        def alm_read_int(file):
                return struct.unpack('>i', file.read(4))[0]

        def alm_read_uint(file):
                return struct.unpack('>I', file.read(4))[0]

        def alm_read_float(file):
                return struct.unpack('>f', file.read(4))[0]

        def alm_read_mat4(file):
                lists = [[], [], [], []]
                for i in range(4):
                        for j in range(4):
                                lists[j].append(alm_read_float(file))
                return Matrix(lists[0], lists[1], lists[2], lists[3])

        def alm_read_vector_element(file):
                e = AlmVectorElement()
                e.bone = alm_read_uint(file)
                e.values = Vector([alm_read_float(file),
                                   alm_read_float(file),
                                   alm_read_float(file),
                                   alm_read_float(file)])
                return e

        def alm_read_vector(file):
                vec = AlmVector()
                count = alm_read_uint(file)
                for j in range(count):
                        vec.elements.append(alm_read_vector_element(file))
                return vec

        def project(vec, transforms):
                sum = Vector(0.0, 0.0, 0.0, 0.0)
                for el in vec.elements:
                        sum = sum + VecMultMat(el.values, transforms[el.bone])
                return sum

        def get_transforms(obj, file, props, frame):
                frame = frame % props.num_frames
                if frame < 0: frame += props.num_frames
                arm = obj.getParent()
                if arm is not None and arm.getType() != 'Armature': arm = None
                if arm is not None:
                        obj.makeDisplayList()
                        arm.makeDisplayList()
                        arm_matrices = armature_to_matrices(arm.data)

                # Extract connectivity
                parents = []
                file.seek(props.bones_offset)
                for i in range(props.num_bones):
                        parents.append(alm_read_uint(file))
                        file.seek(220, 1)  # Skip rest of bone

                # Read local frame matrices
                alm_matrices = []
                file.seek(props.frames_offset + frame * props.num_bones * 16 * 4)
                for i in range(props.num_bones):
                        alm_matrices.append(alm_read_mat4(file))

                # Adjust for skeleton
                for b in range(1, props.num_bones):
                        if arm is not None:
                                alm_matrices[b] = alm_matrices[b] * inverse(arm_matrices.bones[b - 1].relativeRestMatrix())
                                alm_matrices[b] = alm_matrices[b] * arm_matrices.bones[b - 1].relativePosedMatrix()

                # Convert relative to absolute
                for b in range(1, props.num_bones):
                        if parents[b] > 0:
                                alm_matrices[b] = alm_matrices[b] * alm_matrices[parents[b]]

                return alm_matrices

        frame = Blender.Get('curframe') - 1
        if obj.getType() == 'MBall':
                mesh_obj = obj.getParent()
                props = Properties(mesh_obj)
                file = open(os.path.expanduser(os.path.expandvars(props.filename)), "rb")

                transforms = get_transforms(mesh_obj, file, props, frame)
                model_transform = transforms[0]
                transforms[0].identity()
                origin = [0.0, 0.0, 0.0, 0.0]
                for p in obj.getAllProperties():
                        fields = string.split(p.getName(), '_')
                        if fields[0] == 'almcoord':
                                item = int(fields[1])
                                bone = item / 4
                                element = item % 4
                                value = float(p.getData())
                                for i in range(4):
                                        origin[i] += value * transforms[bone][element][i]
                obj.setMatrix(Matrix([1.0, 0.0, 0.0, 0.0],
                                     [0.0, 1.0, 0.0, 0.0],
                                     [0.0, 0.0, 1.0, 0.0],
                                     origin))

        else:
                mesh = obj.getData()
                props = Properties(obj)
                file = open(os.path.expanduser(os.path.expandvars(props.filename)), "rb")

                transforms = get_transforms(obj, file, props, frame)
                model_transform = transforms[0]
                transforms[0].identity()
                file.seek(props.positions_offset)
                for v in mesh.verts:
                        p = alm_read_vector(file)
                        co = project(p, transforms)
                        for i in range(3):
                                v.co[i] = co[i]

                mesh.update(1)

                if props.animate_model:
                        arm = obj.getParent()
                        if arm is not None and arm.getType() != 'Armature': arm = None
                        if arm is not None: arm.setMatrix(model_transform)
                        else: obj.setMatrix(model_transform)

do_work(Blender.link)
