#!BPY

""" Registration info for Blender menus:
Name: 'Armature info'
Blender: 235
Group: 'Object'
Tip: 'GUI to view matrices and quaternions for bones'
"""

__author__ = "Bruce Merry"
__version__ = "1"

import Blender
from Blender import *

menu = None

def event_handler(event, val):
        Draw.Redraw()

def button_handler(event):
        if (event == 1): Blender.Draw.Exit()
        Draw.Redraw()

def text_at(text, x, y):
        BGL.glRasterPos2i(x, y)
        Draw.Text(text)

def draw_vector(vec, x, y):
        for i in vec:
                x += 40
                text = "%.2f" % i
                width = Draw.GetStringWidth(text)
                text_at(text, x - width, y)

def draw_matrix(mat, x, y):
        for row in mat:
                draw_vector(row, x, y)
                y -= 15

def draw_handler():
        global menu
        BGL.glClearColor(0, 0.5, 0.5, 1)
        BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
        BGL.glColor3f(1, 1, 1)
        Draw.Button("Exit", 1, 20, 1, 80, 20)

        if not Object.GetSelected(): return
        obj = Object.GetSelected()[0]
        if obj.getType() != 'Armature': return
        x = 20
        y = 140
        bones = obj.data.getBones()

        if menu is None: selected = 0
        else: selected = menu.val
        if selected < 0 or selected >= len(bones): selected = 0
        bone = bones[selected]
        draw_matrix(bone.getRestMatrix('worldspace'), x, y - 15)
        draw_matrix(bone.getRestMatrix('bonespace'), x + 170, y - 15)
        draw_vector(bone.getQuat(), x + 340, y - 15)
        draw_matrix(bone.getQuat().toMatrix(), x + 340, y - 30)
        draw_vector(bone.getHead(), x, y - 80)
        draw_vector(bone.getTail(), x + 170, y - 80)
        text_at(`bone.getRoll()`, x + 340, y - 80)
        if bone.getParent() is not None:
                text_at("Parent: %s" % bone.getParent(), x, y - 100)
        else:
                text_at("Root", x, y - 100)

        menu_text = "BONE %t"
        for i in range(len(bones)):
                menu_text += "|%s %%x%d" % (bones[i].name, i)
        menu = Draw.Menu(menu_text, 2, x, y + 10, 100, 20, selected)

Blender.Draw.Register(draw_handler, event_handler, button_handler)
