#!BPY

""" Registration info for Blender menus:
Name: 'Armature info'
Blender: 240
Group: 'Animation'
Tip: 'GUI to view matrices and quaternions for bones'
"""

__author__ = "Bruce Merry"
__version__ = "2"

import Blender
from Blender import *

bone_menu = None
space_menu = None
posed_menu = None

def event_handler(event, val):
        Draw.Redraw()

def button_handler(event):
        if (event == 1): Blender.Draw.Exit()
        Draw.Redraw()

def text_at(text, x, y):
        BGL.glRasterPos2i(x, y)
        Draw.Text(text)

def draw_vector(vec, x, y):
        for i in vec:
                x += 40
                text = "%.2f" % i
                width = Draw.GetStringWidth(text)
                text_at(text, x - width, y)

def draw_matrix(mat, x, y):
        for row in mat:
                draw_vector(row, x, y)
                y -= 15

def draw_handler():
        global bone_menu, space_menu, posed_menu
        BGL.glClearColor(0, 0.5, 0.5, 1)
        BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
        BGL.glColor3f(1, 1, 1)
        Draw.Button("Exit", 1, 20, 1, 80, 20)

        if not Object.GetSelected(): return
        obj = Object.GetSelected()[0]
        if obj.getType() != 'Armature': return
        x = 20
        y = 140

        if bone_menu is None: selected = 0
        else: selected = bone_menu.val
        if space_menu is None: space = 0
        else: space = space_menu.val
        space_name = ('ARMATURESPACE', 'BONESPACE')[space]
        if posed_menu is None: posed = 0
        else: posed = posed_menu.val

        if posed:
                bones = obj.getPose().bones.values()
        else:
                bones = obj.data.bones.values()
        if selected < 0 or selected >= len(bones): selected = 0
        bone = bones[selected]

        if posed:
                draw_matrix(bone.poseMatrix, x, y - 15)
                draw_vector(bone.loc, x, y - 80)
                draw_vector(bone.size, x + 170, y - 80)
        else:
                draw_matrix(bone.matrix[space_name], x, y - 15)
                draw_vector(bone.head[space_name], x, y - 80)
                draw_vector(bone.tail[space_name], x + 170, y - 80)
                text_at(`bone.roll[space_name]`, x + 340, y - 80)
                if bone.hasParent():
                        text_at("Parent: %s" % bone.parent.name, x, y - 100)
                else:
                        text_at("Root", x, y - 100)

        menu_text = "Bone %t"
        for i in range(len(bones)):
                menu_text += "|%s %%x%d" % (bones[i].name, i)
        bone_menu = Draw.Menu(menu_text, 2, x, y + 10, 100, 20, selected)

        menu_text = "Space %t|Armature %x0|Bone %x1"
        space_menu = Draw.Menu(menu_text, 3, x + 120, y + 10, 100, 20, space)

        menu_text = "Pose %t|Rest %x0|Current %x1"
        posed_menu = Draw.Menu(menu_text, 4, x + 240, y + 10, 100, 20, posed)

Draw.Register(draw_handler, event_handler, button_handler)
