import Blender, math
from Blender import *
from Blender.Mathutils import *

# Vertex things
class AsVectorElement:
        def __init__(self):
                self.bone = -1
                self.values = Vector([0.0, 0.0, 0.0, 0.0])


class AsVector:
        def __init__(self):
                self.elements = []

def value_map(x):
        return x / 1048576.0

def value_unmap(x):
        return x * 1048576.0

def get_influence_coords(mesh, name, restrict = None):
        coords = {}

        try:
                for d, prefix in enumerate(['X.', 'Y.', 'Z.', 'W.']):
                        for sign, lbl in [(-1, '-'), (1, '+')]:
                                if restrict is not None:
                                        influences = mesh.getVertsFromGroup(lbl + prefix + name, 1, restrict)
                                else:
                                        influences = mesh.getVertsFromGroup(lbl + prefix + name, 1)
                                for v, w in influences:
                                        if not coords.has_key(v):
                                                coords[v] = Vector(0.0, 0.0, 0.0, 0.0)
                                        coords[v][d] += sign * value_unmap(w)
        except AttributeError:
                return {}  # Can happen if the group is empty
        return coords

def set_influence_coords(mesh, name, coords):
        for d, prefix in enumerate(['X.', 'Y.', 'Z.', 'W.']):
                for v, c in coords.iteritems():
                        mesh.assignVertsToGroup('+' + prefix + name, [v], value_map(max(c[d], 0.0)), Mesh.AssignModes['REPLACE'])
                        mesh.assignVertsToGroup('-' + prefix + name, [v], value_map(-min(c[d], 0.0)), Mesh.AssignModes['REPLACE'])

def add_influence_groups(mesh, name):
        for prefix in ['X.', 'Y.', 'Z.', 'W.']:
                for lbl in ['-', '+']:
                        mesh.addVertGroup(lbl + prefix + name)

# Matrix things
def extend(v, w):
        return Vector([v[0], v[1], v[2], w])

def contract(v):
        return Vector([v[0], v[1], v[2]])

def inverse(m):
        mat = Matrix(m)
        return mat.invert()

def project(vec, transforms):
        sum = Vector(0.0, 0.0, 0.0, 0.0)
        for el in vec.elements:
                sum = sum + el.values * transforms[el.bone]
        return sum

# Armature things
def bone_name(i):
        if i == 0: return "Root"
        return "Bone.%03d" % (i);

def rel_rest_matrix(bone):
        if bone.hasParent():
                return bone.matrix['ARMATURESPACE'] * inverse(bone.parent.matrix['ARMATURESPACE'])
        else:
                return bone.matrix['ARMATURESPACE']

# General things
def get_version(obj):
        try:
                try:
                        prop = obj.getProperty("as_version")
                        return int(prop.getData())
                except RuntimeError:
                        return 0
        except AttributeError:
                return 0

def set_version(obj, version):
        try:
                try:
                        prop = obj.getProperty("as_version")
                        prop.setData(version)
                        return
                except RuntimeError:
                        pass
        except AttributeError:
                pass
        obj.addProperty("as_version", version, 'INT')
