#!BPY

""" Registration info for Blender menus:
Name: 'Export camera path (ALOD)'
Blender: 242
Group: 'Object'
Tip: 'Exports the positions and orientations of an object over time'
"""

import Blender;
from Blender.Mathutils import *;

editmode = Blender.Window.EditMode()    # are we in edit mode?  If so ...
if editmode: Blender.Window.EditMode(0) # leave edit mode before getting the mesh

def fs_callback(filename):
	file = open(filename, "wb")
        scene = Blender.Scene.GetCurrent()
        context = scene.getRenderingContext()
        object = scene.getActiveObject()
        file.write("<camerapath>\n")
        for f in range(context.startFrame(), context.endFrame() + 1):
                object.evaluatePose(f)
                q = object.getMatrix('worldspace').toQuat()
                trans = object.getMatrix('worldspace').translationPart()
                file.write("    <cameraframe>\n")
                file.write("        <translate offset=\"%.6f %.6f %.6f\" />\n" % (trans.x, trans.y, trans.z))
                file.write("        <rotate axis=\"%.6f %.6f %.6f\" angle=\"%.6f\" />\n" % (q.axis.x, q.axis.y, q.axis.z, q.angle))
                file.write("    </cameraframe>\n")
        file.write("</camerapath>\n")
        file.close()
        if editmode: Blender.Window.EditMode(1)

def create_path():
        filename = Blender.Get('filename')
        if filename.find('.') != -1:
                return filename.split('.')[0] + ".xml"
        return filename


Blender.Window.FileSelector(fs_callback, "Export camera path", create_path());
