#!/usr/bin/perl -w
use strict;
use warnings;
use DBI;

my $source_dir = $ENV{HOME} . '/uct/msc/models/videos/';

$ENV{'LD_PRELOAD'} = 'libbugle.so';
$ENV{'BUGLE_CHAIN'} = 'alod_video';
$ENV{'__GL_FSAA_MODE'} = 4;       # 4x bilinear for GF5-8
# Breaks dynamic shading
# $ENV{'__GL_LOG_MAX_ANISO'} = 2;   # 4x aniso

my $dbh = DBI->connect('dbi:mysql:mysql_read_default_group=meshexp', undef, undef,
                       { RaiseError => 1, AutoCommit => 1 })
    or die "Failed to connect: $DBI::errstr";

my $st = $dbh->prepare('SELECT scene, memoryless, as_metric, static_normals, influences, tolerance, filename FROM tests');
$st->execute;
my $rows = $st->fetchall_arrayref;
foreach my $row (@$rows)
{
    my $xml_name = $row->[0];
    my @shaders = ('SSE by vertex (buffered)', 'Dot product');
    if (!$row->[1]) { $xml_name .= '-mem'; }
    if (!$row->[2]) { $xml_name .= '-3d'; }
    if (!$row->[3]) { push @shaders, 'Dynamic pixel (8x8)'; }
    else { push @shaders, 'Static pixel'; }
    if (!$row->[4]) { $xml_name .= '-noinf'; }
    my $target = $ENV{'ALOD_VIDEOS'} . '/' . $row->[6];
    $xml_name = $source_dir . $xml_name . '.xml';
    if (-f $target) { next; }
    if (!-f $xml_name)
    {
        warn "Skipping $target because $xml_name does not exist.\n";
        next;
    }

    my @args = ('--camera', $source_dir . 'spiral.xml', '--initial-lod=' . $row->[5],
                '--width', 1280, '--height', 1024,
                '--initial-shaders=' . join(',', @shaders));
    unlink('alod.avi');
    if (-f 'alod.avi') { die "Could not delete alod.avi: $!"; }
    print "Generating $target\n";
    system('./render/meshrender', @args, $xml_name) == 0 or die "meshrender failed: $?";
    if (!-f 'alod.avi') { die "No output video"; }

    system('/bin/mv', '--', 'alod.avi', $target) == 0 or die("mv failed: $?");
}

$dbh->disconnect();
