#!/usr/bin/perl -w
use strict;
use warnings;

my @vertices = ();
my @normals = ();
my @faces = ();
while (<>)
{
    if (/^v\s+([-.\d]+)\s+([-.\d]+)\s+([-.\d]+)\s+$/)
    {
        push @vertices, [$1, $2, $3];
    }
    elsif (/^vn\s+([-.\d]+)\s+([-.\d]+)\s+([-.\d]+)\s+$/)
    {
        push @normals, [$1, $2, $3];
    }
    elsif (m|^f\s+([-.\d]+)//[-.\d]+\s+([-.\d]+)//[-.\d]+\s+([-.\d]+)//[-.\d]+|)
    {
        push @faces, [$1 - 1, $2 - 1, $3 - 1];
    }
}

my $vertices = scalar(@vertices);
my $faces = scalar(@faces);
print <<EOF;
ply
format ascii 1.0
comment create by horse2ply
element vertex $vertices
property float32 x
property float32 y
property float32 z
property float32 nx
property float32 ny
property float32 nz
property float32 s
property float32 t
element face $faces
property list uint8 int32 vertex_indices
end_header
EOF

for (my $i = 0; $i < $vertices; $i++)
{
    my $x = ($i & 255) / 256.0;
    my $y = ($i >> 8) / 256.0;
    printf("%.9f %.9f %.9f  %.9f %.9f %.9f  %.9f %.9f\n",
        $vertices[$i]->[0], $vertices[$i]->[1], $vertices[$i]->[2],
        $normals[$i]->[0],  $normals[$i]->[1],  $normals[$i]->[2],
        $x, $y);
}
foreach my $f (@faces)
{
    printf "3 %d %d %d\n", $f->[0], $f->[1], $f->[2];
}

