#!/usr/bin/perl -w
use strict;
use warnings;
use DBI;
use Getopt::Long;

my ($test_st, $sample_st);

my $model_dir = $ENV{'HOME'} . '/uct/msc/models/videos/';

sub add_test
{
    my @nameopts = @_;
    $nameopts[5] = sprintf("%.3f", $_[5]);
    my $filename = join('_', @nameopts) . '.avi';
    $test_st->execute(@_, 0.0, $filename);
}

sub get_filename($$)
{
    my ($scene, $opts) = @_;
    my $file = $scene;
    if (!$opts->[0]) { $file .= '-mem'; }
    if (!$opts->[1]) { $file .= '-3d'; }
    if (!$opts->[3]) { $file .= '-noinf'; }
    return $model_dir . $file . '.xml';
}

sub equiv_lod($$$$)
{
    my ($scene, $opts1, $opts2, $lod1) = @_;

    my $file1 = get_filename($scene, $opts1);
    my $file2 = get_filename($scene, $opts2);
    if ($file1 eq $file2) { return $lod1; }
    my $camera = $model_dir . 'spiral.xml';

    open(my $pipe, '-|', 'render/meshrender', '--show-influences=' . $lod1, $file1, '--camera', $camera) or die;
    my $influences = <$pipe>;
    chomp($influences);
    close($pipe) or die;

    open($pipe, '-|', 'render/meshrender', '--show-influences=' . (-$influences), $file2, '--camera', $camera) or die;
    my $lod2 = <$pipe>;
    chomp($lod2);
    close($pipe) or die;
    return $lod2;
}

sub add_sample
{
    my $experiment = shift;
    my $scene = shift;
    my @test1 = (shift, shift, shift, shift, shift);
    my @test2 = (shift, shift, shift, shift, shift);
    add_test($scene, @test1);
    add_test($scene, @test2);
    $sample_st->execute($experiment, $scene, @test1, @test2, undef, 1);
    $sample_st->execute($experiment, $scene, @test1, @test2, 0, 0);
    $sample_st->execute($experiment, $scene, @test1, @test2, 1, 0);
}

my @e1tols = (0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4..8);
my %scenes =
    ('cat' => [1.5, 3.0, 6.0],
     'horse' => [1.5, 3.0, 6.0],
     'cyberdemon' => [1.5, 3.0, 6.0],
     'arm' => [1.5, 3.0, 6.0],
     'mancandy' => [1.5, 3.0, 6.0]
    );
my @scenes = keys %scenes;

my @do_experiments = ();
GetOptions("experiments=s" => \@do_experiments) or die;
@do_experiments = split(/,/, join(',', @do_experiments));

my $dbh = DBI->connect('dbi:mysql:mysql_read_default_group=meshexp', undef, undef,
                       { RaiseError => 1, AutoCommit => 1 })
    or die "Failed to connect: $DBI::errstr";

$test_st = $dbh->prepare('INSERT INTO tests VALUES(?, ?, ?, ?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE filename=VALUES(filename)');
$sample_st = $dbh->prepare('INSERT INTO samples VALUES(NULL, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE swap=VALUES(swap)');

foreach my $scene (@scenes)
{
    my @scenelods = @{$scenes{$scene}};

    # Experiment 1: LOD vs non-LOD
    if (grep { $_ == 1 } @do_experiments)
    {
        foreach my $tol (@e1tols)
        {
            my @test1 = (1, 1, 1, 1, $tol);
            my @test2 = (1, 1, 1, 1, 0);
            add_sample(1, $scene, @test1, @test2);
        }
    }

    foreach my $l (@scenelods)
    {
        my %tests = (
            2 => [[1, 1, 1, 0], [1, 0, 1, 0]],     # AS vs 3D metric
            3 => [[1, 1, 1, 1], [1, 1, 0, 1]],     # Static vs dynamic normals
            4 => [[1, 1, 1, 1], [1, 1, 1, 0]]      # Infsimp/noinfsimp
        );
        foreach my $e (grep { exists($tests{$_}) } @do_experiments)
        {
            my $l2 = equiv_lod($scene, $tests{$e}->[0], $tests{$e}->[1], $l);
            add_sample($e, $scene, @{$tests{$e}->[0]}, $l, @{$tests{$e}->[1]}, $l2);
        }
    }
}

$dbh->disconnect();
