#!/usr/bin/perl -w
use strict;
use warnings;
use File::Path;
use IO::Handle;
use Sys::Hostname;

my %methods = (
    'fploop' => 'FP with loop',
    'vertextex' => 'Vertex textures',
    'ssevb' => 'SSE by vertex (buffered)',
    'ssevs' => 'SSE by vertex (streamed)',
    'sseb' => 'SSE by bone'
);
my $modeldir = $ENV{'HOME'} . '/uct/msc/models/results/renderspeed/';
my $outputdir = $ENV{'HOME'} . '/uct/msc/analyses/renderspeed/' . hostname() . '/';
my $camerapath = $modeldir . 'away.xml';

while (my ($mname, $mshader) = each(%methods))
{
    open(LOG, '>', $outputdir . $mname . '.txt') or die;
    print LOG "Objects\tVertices\tInfluences\tTime\n";
    my $shaders = 'DirectLOD,NoFrustumCull,Null,Normal,' . $mshader;
    my @models = qw(arm horse cat cyberdemon mancandy);
    if ($mname eq 'vertextex') { pop @models; }
    @models = map { ($_, $_ . '-inf') } @models;
    @models = map { ($_ . '.alm', $_ . '.xml') } @models;
    @models = map { $modeldir . $_ } @models;
    foreach my $model (@models)
    {
        my $opts = "--initial-shaders='$shaders' -W 1280 -H 1024 -c '$camerapath'";
        my $influences = `render/meshrender $opts --show-influences $model`;

        my %influences = ();
        while ($influences =~ /^(\d+)\s+([\d.]+)\s+([\d.]+)\s+([\d.]+)$/gm)
        {
            $influences{$1} = [$2, $3, $4];
        }

        for (my $l = 0; $l <= 5; $l++)
        {
            my $time = `render/meshrender $opts --initial-lod=$l -v $model`;
            $time =~ /: ([\d.]+)s/ or die("Did not find rendering time");
            print LOG join("\t", @{$influences{$l}}), "\t$1\n";
        }
        print STDERR "Done $model with $mname\n"
    }
    close(LOG);
}

