#!/usr/bin/perl -w
use strict;
use warnings;
use List::Util qw(min max);

my %data = ();
while (<>)
{
    chomp;
    my @fields = split('\t', $_);
    my $model = $fields[0];
    my $method = $fields[1];
    my $perfrag = $fields[4];
    $data{$model}->{$method} = $perfrag;
}

print '\begin{tabular}{l|D{.}{.}{4.1}|D{.}{.}{4.1}|*{5}{D{.}{.}{5.1}}|D{.}{.}{3.2}D{.}{.}{3.2}}', "\n";
print <<'EOF';
\hline
\multicolumn{1}{c|}{\textbf{Model}}&
\multicolumn{1}{c|}{\textbf{Vertex}}&
\multicolumn{1}{c|}{\textbf{Static}}&
\multicolumn{5}{c|}{\textbf{Dynamic}}&
\multicolumn{1}{c}{\textbf{Static}}&
\multicolumn{1}{c}{\textbf{Dynamic}}\\
\multicolumn{1}{c|}{}&
\multicolumn{1}{c|}{}&
\multicolumn{1}{c|}{}&
EOF
for (my $t = 2; $t <= 32; $t *= 2)
{
    my $bar = ($t == 32) ? '|' : '';
    print "\\multicolumn{1}{c$bar}{\$\\mathbf{$t\\times $t}\$}&\n";
}
print <<'EOF';
\multicolumn{1}{c}{\textbf{Ratio}}&
\multicolumn{1}{c}{\textbf{Ratio}}\\
\hline
EOF
my @models = qw(horse cat arm cylinder mancandy cyberdemon);
foreach my $model (@models)
{
    my $d = $data{$model};
    my $vertex = $d->{'Vertex'};
    my $static = $d->{'Static pixel'};
    my @dynamic = ();
    for (my $t = 2; $t <= 32; $t *= 2)
    {
        push @dynamic, $d->{'Dynamic pixel (' . $t . 'x' . $t . ')'};
    }
    my $best_dynamic = min(@dynamic);
    my $static_ratio = $static / $vertex;
    my $dynamic_ratio = $best_dynamic / $vertex;
    printf("%-15s & %6.1f & %6.1f & ", $model, $vertex, $static);
    foreach (@dynamic)
    {
        if ($_ == $best_dynamic)
        {
            printf("\\multicolumn{1}{b{5.1}%s}{%7.1f} &", 
                ($_ == $dynamic[$#dynamic]) ? '|' : '', $_);
        }
        else
        {
            printf("%7.1f & ", $_);
        }
    }
    printf("%6.2f & %6.2f\\\\\n",
        $static_ratio, $dynamic_ratio);
}
print <<'EOF';
\hline
\end{tabular}
EOF
