#!/usr/bin/perl -w
use strict;
use warnings;

my $renderer = $ENV{'HOME'} . '/uct/msc/code/alod-release/render/meshrender';
my $models = $ENV{'HOME'} . '/uct/msc/models/videos';
my @common = ('--initial-lod', '100', '--camera', $models . '/spiral.xml');
my @common_shaders = ('DirectLOD', 'SSE by vertex (buffered)', 'Dot product');

sub grab(@)
{
    open(my $fh, '-|', @_) or die("Failed to open $_[0]: $!");
    local $/;
    $/ = undef;
    my $output = <$fh>;
    defined($output) or die("Failed to read from $_[0]: $!");
    close($fh) or die("Failed to close pipe: $!");
    return $output;
}

sub get_frags($)
{
    my $model_file = shift;
    local $ENV{'LD_PRELOAD'};
    local $ENV{'BUGLE_CHAIN'};
    $ENV{'LD_PRELOAD'} = 'libbugle.so';
    $ENV{'BUGLE_CHAIN'} = 'alod_samples';
    my $log = grab($renderer, @common, '--initial-shaders=' . join(',', @common_shaders, 'Vertex'), $model_file);
    my $frags = 0.0;
    my $frames = 0;
    while ($log =~ /^(\d+) fragments\/frame/g)
    {
        $frags += $1;
        $frames++;
    }
    return $frags / $frames;
}

foreach my $model ('horse', 'cat', 'arm', 'cylinder', 'mancandy', 'cyberdemon')
{
    my $model_file = "$models/$model.xml";
    my $frags = get_frags($model_file);
    foreach my $method ('Vertex', 'Static pixel', 'Dynamic pixel (2x2)', 'Dynamic pixel (4x4)', 'Dynamic pixel (8x8)', 'Dynamic pixel (16x16)', 'Dynamic pixel (32x32)')
    {
        my $time = grab($renderer, '-v', @common, '--initial-shaders=' . join(',', @common_shaders, $method), $model_file);
        $time =~ /Time per frame: ([\d.]+)s$/ or die "Time not found in $time";
        $time = $1;
        printf("%s\t%s\t%.3f\t%d\t%.3f\n", $model, $method, $time * 1000, $frags, $time / $frags * 1e9);
    }
}
