#!/usr/bin/perl
use strict;
use warnings;
use DBI;

sub get_all($$)
{
    my ($dbh, $col) = @_;
    my $ref = $dbh->selectall_arrayref("SELECT $col FROM ordered1 GROUP BY $col ORDER BY $col");
    return map { $_->[0] } @$ref;
}

my $dbh = DBI->connect('dbi:mysql:mysql_read_default_group=meshexp', undef, undef,
    { RaiseError => 1, AutoCommit => 1})
    or die "Failed to connect: $DBI::errstr";

my @tols = get_all($dbh, 'tol');
my @scenes = get_all($dbh, 'scene');
my @subjects = get_all($dbh, 'subject');
my $scene_label_row = int(scalar(@subjects) / 2);

print <<'EOF';
% Automatically generated by tabexp1.pl - do not edit
\centering
\begin{tiny}
EOF
print '\\begin{tabularx}{\textwidth}{|l|l|', 'C' x scalar(@tols), '|}', "\n";
print '\\hline', "\n";
print '\\textbf{Scene} & \\textbf{Subject ID} & \\multicolumn{', scalar(@tols), '}{c|}{\textbf{Pixel tolerance}}\\\\', "\n";
print '&&', join('&', @tols), "\\\\\n\\hline\n";

my $st_subject = $dbh->prepare('SELECT picked FROM ordered1 WHERE scene=? AND subject=? AND tol=? ORDER BY swapped');
my $st_scene = $dbh->prepare('SELECT SUM(picked) FROM ordered1 WHERE scene=? AND tol=? GROUP BY scene, tol');
my $st_total = $dbh->prepare('SELECT SUM(picked) FROM ordered1 WHERE tol=? GROUP BY tol');
foreach my $scene (@scenes)
{
    my $scene_row = 0;
    foreach my $subject (@subjects)
    {
        print(($scene_row == $scene_label_row) ? $scene : '');
        print ' & ', $subject;
        foreach my $tol (@tols)
        {
            print ' & ';
            my $ref = $dbh->selectall_arrayref($st_subject, {}, $scene, $subject, $tol);
            foreach my $row (@$ref)
            {
                print $row->[0] ? '$\\bullet$ ' : '$\\circ$ ';
            }
        }
        $scene_row++;
        print '\\\\', "\n";
    }
    print ' & Total (/34)';
    foreach my $tol (@tols)
    {
        my $row = $dbh->selectrow_arrayref($st_scene, {}, $scene, $tol);
        print ' & ', $row->[0];
    }
    print "\\\\\n\\hline\n";
}
print "Total (/170) &";
foreach my $tol (@tols)
{
    my $row = $dbh->selectrow_arrayref($st_total, {}, $tol);
    print ' & ', $row->[0];
}
print <<'EOF';
\\
\hline
\end{tabularx}\end{tiny}
EOF

