#!/usr/bin/perl
use strict;
use warnings;
use DBI;
use Data::Dumper;

my $experiment = $ARGV[0] or die;

sub get_all($$)
{
    my ($dbh, $col) = @_;
    my $ref = $dbh->selectall_arrayref("SELECT $col FROM results join samples on sample=samples.id WHERE experiment=$experiment GROUP BY $col ORDER BY $col");
    return map { $_->[0] } @$ref;
}

my $dbh = DBI->connect('dbi:mysql:mysql_read_default_group=meshexp', undef, undef,
    { RaiseError => 1, AutoCommit => 1})
    or die "Failed to connect: $DBI::errstr";

my @tols = get_all($dbh, 'test1_tolerance');
my @scenes = get_all($dbh, 'scene');
my @subjects = get_all($dbh, 'subject');
my $tol_label_row = int(scalar(@subjects) / 2);

my $st_subject = $dbh->prepare("SELECT preferred1, 1 - priority FROM results JOIN samples ON sample=samples.id WHERE scene=? AND subject=? AND test1_tolerance=? AND experiment=$experiment ORDER BY priority DESC, swap ASC");
my $st_tol = $dbh->prepare("SELECT SUM(preferred1), COUNT(preferred1) FROM results JOIN samples ON sample=samples.id WHERE scene=? AND test1_tolerance=? AND experiment=$experiment AND priority=0 GROUP BY test1_tolerance, scene");
my $st_total = $dbh->prepare("SELECT SUM(preferred1), COUNT(preferred1) FROM results JOIN samples ON sample=samples.id WHERE scene=? AND experiment=$experiment AND priority=0 GROUP BY scene");


sub do_row($$@)
{
    my $st = shift;
    my $dots = shift;

    my $A = 0;
    my $B = 0;
    foreach my $scene (@scenes)
    {
        print ' & ';
        my $ref = $dbh->selectall_arrayref($st, {}, $scene, @_,);
        my $first = 1;
        foreach my $row (@$ref)
        {
            if ($row->[1])
            {
                $A += $row->[0];
                $B += $row->[1];
            }
            if ($dots)
            {
                if ($first) { print "{\\lightgray "; }
                print $row->[0] ? '$\\bullet$ ' : '$\\circ$ ';
                if ($first) { print "}"; }
            }
            else
            {
                print "$row->[0]/$row->[1]";
            }
            $first = 0;
        }
    }
    print " & $A/$B";
    print '\\\\', "\n";
}

print <<'EOF';
% Automatically generated by tabexp2.pl - do not edit
\centering
\begin{tiny}
EOF
print '\\begin{tabular}{|l|l|', 'c' x scalar(@scenes), '|D{/}{/}{-1}|}', "\n";
print '\\hline', "\n";
print '\\textbf{Pixel} & \\textbf{Subject} & \\multicolumn{', scalar(@scenes), '}{c|}{\textbf{Scene}} & \multicolumn{1}{c|}{\textbf{Total}}\\\\', "\n";
print '\cline{3-7}', "\n";
print '\\textbf{tolerance} & \textbf{ID} &', join('&', @scenes), "&\\\\\n\\hline\n";

foreach my $tol (@tols)
{
    my $tol_row = 0;
    foreach my $subject (@subjects)
    {
        print(($tol_row == $tol_label_row) ? $tol : '');
        print ' & ', $subject;
        do_row($st_subject, 1, $subject, $tol);
        $tol_row++;
    }
    print ' & ';
    do_row($st_tol, 0, $tol);
    print "\\hline\n";
}
print "Total &";
do_row($st_total, 0);
print <<'EOF';
\hline
\end{tabular}\end{tiny}
EOF
