#!/usr/bin/perl
use strict;
use warnings;

my @methods = ('ssevb', 'sseb', 'fploop', 'vertextex');
my @captions = (
                ['', '', '', '', 'Alternate', 'Fragment', 'Vertex'],
                ['Objects', 'Vertices', 'Influences', 'CPU', 'CPU', 'Program', 'Textures']
               );

my @fh;
for my $method (@methods)
{
    open(my $f, '<', "$method.txt") or die;
    readline($f); # Eliminates header
    push @fh, $f;
}

my @lines;
while (defined($_ = readline($fh[0])))
{
    my @fields = split;
    my $line = sprintf("%d. & %d. & %d.", @fields[0..2]);
    my @times = (sprintf("%.3f", $fields[3] * 1000));

    for (my $i = 1; $i < @fh; $i++)
    {
        if (defined($_ = readline($fh[$i])))
        {
            @fields = split;
            push @times, sprintf("%.3f", $fields[3] * 1000);
        }
        else
        {
            push @times, '\multicolumn{1}{c}{---}';
        }
    }
    $line = join(' & ', $line, @times);
    push @lines, $line;
}

print <<'EOF';
\begin{tabular}{w{3}w{5}w{5}*{4}{D{.}{.}{1.3}}|w{3}w{5}w{5}*{4}{D{.}{.}{1.3}}}
\hline
EOF
for (my $i = 0; $i < 2; $i++)
{
    my @fields = map{ "\\multicolumn{1}{c}{\\textbf{$_}}" } (@{$captions[$i]}, @{$captions[$i]});
    $fields[6] =~ s/\{c\}/{c|}/;
    print join(' & ', @fields), "\\\\\n";
}
print "\\hline\n";

my $mid = int((scalar(@lines) + 1) / 2);
my @lines1 = @lines[0..($mid-1)];
my @lines2 = @lines[$mid..$#lines];
if (scalar(@lines2) < scalar(@lines1))
{
    push @lines2, '';
}
for (my $i = 0; $i < scalar(@lines1); $i++)
{
    print "$lines1[$i] & $lines2[$i]\\\\\n";
}

print <<'EOF';
\hline
\end{tabular}
EOF
