#!BPY

""" Registration info for Blender menus:
Name: 'Materials for UV patches'
Blender: 237
Group: 'Materials'
Tip: 'Create a separate material index for each patch determined by UV mapping'
"""

import Blender;
import meshtools;

objects = Blender.Object.GetSelected()
meshname = objects[0].data.name
mesh = Blender.NMesh.GetRaw(meshname)

edge_table = meshtools.generate_edgetable(mesh)
edge_keys = edge_table.keys()
edge_keys.sort()

seen_faces = {}
mat_index = 0
mats = []
for i in range(len(mesh.faces)):
        if (not seen_faces.has_key(i)):
                queue = [i]
                mats.append(None)
                seen_faces[i] = 1;
                while len(queue):
                        face = queue.pop(0)
                        mesh.faces[face].materialIndex = mat_index;
                        for j in range(len(mesh.faces[face].v)):
                                key1 = (mesh.faces[face].v[j - 1].index, mesh.faces[face].v[j].index)
                                uv1 = (mesh.faces[face].uv[j - 1], mesh.faces[face].uv[j])
                                if (edge_table.has_key(key1)):
                                        next = edge_table[key1][0]
                                else:
                                        next = edge_table[(key1[1], key1[0])][1]
                                if next is None:
                                        print "Missing due for", face
                                valid = 0
                                for k in range(len(mesh.faces[next].v)):
                                        key2 = (mesh.faces[next].v[k].index, mesh.faces[next].v[k - 1].index)
                                        uv2 = (mesh.faces[next].uv[k], mesh.faces[next].uv[k - 1])
                                        if (key1 == key2 and uv1 == uv2):
                                                valid = 1
                                                break
                                if valid and not seen_faces.has_key(next):
                                        queue.append(next)
                                        seen_faces[next] = 1
                mat_index += 1

print "Found %d materials\n" % len(mats)
mesh.setMaterials(mats)
mesh.update()
