#!BPY

""" Registration info for Blender menus:
Name: 'Seams from UV patches'
Blender: 242
Group: 'UV'
Tip: 'Automatically fills in seams from discontinuities in the existing UV-mapping'
"""

import Blender;

editmode = Blender.Window.EditMode()    # are we in edit mode?  If so ...
if editmode: Blender.Window.EditMode(0) # leave edit mode before getting the mesh

objects = Blender.Object.GetSelected()
mesh = objects[0].getData(False, True)

# Keys are pairs of vertex indices. Values are pairs of UV coordinates for
# the face for which the indices are listed ccw.
uvdict = {}
for f in mesh.faces:
        for j in range(len(f.verts)):
                j2 = (j + 1) % len(f.verts)
                uvdict[(f.verts[j].index, f.verts[j2].index)] = (f.uv[j], f.uv[j2])
for e in mesh.edges:
        v1i = e.v1.index
        v2i = e.v2.index
        uva = uvdict[(v1i, v2i)]
        uvb = uvdict[(v2i, v1i)]
        if (uva[0] - uvb[1]).length > 2e-2 or (uva[1] - uvb[0]).length > 2e-2:
                e.flag |= Blender.Mesh.EdgeFlags.SEAM
mesh.update()

if editmode: Blender.Window.EditMode(1)
