#version 110

`#'define `MODE' MODE
#if MODE == 0
#  define USE_CORRECT
#endif
#if MODE == 1
#  define USE_INVERT
#endif
#if MODE == 2
#  define USE_COMBINE
#endif
#if MODE == 3
#  define USE_CORRECT
#  define USE_INVERT
#endif
#if MODE == 4
#  define USE_CORRECT
#  define USE_COMBINE
#endif

#ifdef USE_CORRECT
varying vec3 n_correct;
#endif
#ifdef USE_INVERT
varying vec3 n_invert;
#endif
#ifdef USE_COMBINE
varying vec3 n_combine;
#endif

varying vec3 eyepos;

const vec3 lightpos = vec3(0.2, 1.0, 1.0);
const float ambient = 0.2;
const float diffuse = 0.4;
const float specular = 0.6;
const float shininess = 128.0;

float dotl(vec3 a, vec3 b)
{
    return max(dot(a, b), 0.0);
}

float light(vec3 normal, vec3 lightpos)
{
    vec3 lightdir = normalize(lightpos - eyepos);
    vec3 half = normalize(mix(lightdir, normalize(-eyepos), 0.5));
    float d = dot(normal, lightdir);
    float s = pow(dotl(normal, half), shininess);
    return (d < 0.0) ? ambient : d * diffuse + s * specular + ambient;
}

float lighting(vec3 normal)
{
    return light(normal, lightpos);
}

#ifdef NORMAL_MAP
vec4 color_lit(vec3 normal)
{
    normal = normalize(normal);
    return 0.5 * vec4(normal, 1.0) + 0.5;
}
#else
vec4 color_lit(vec3 normal)
{
    normal = normalize(normal);
    float lit = lighting(normal);
    return vec4(lit, lit, lit, 1.0);
}
#endif

vec4 color_error(vec3 correct, vec3 normal)
{
    const float step = .34907; /* 20 degrees */

    correct = normalize(correct);
    normal = normalize(normal);
    float error = acos(min(dot(correct, normal), 1.0)) / step;
    float lit = lighting(correct);
    if (error < 1.0)
        return mix(vec4(lit, lit, lit, 1.0), vec4(0.0, lit, 0.0, 1.0), error);
    else if (error < 2.0)
        return mix(vec4(0.0, lit, 0.0, 1.0), vec4(lit, lit, 0.0, 1.0), error - 1.0);
    else if (error < 3.0)
        return mix(vec4(lit, lit, 0.0, 1.0), vec4(lit, 0.0, 0.0, 1.0), error - 2.0);
    else
        return mix(vec4(lit, 0.0, 0.0, 1.0), vec4(lit, 0.0, lit, 1.0), error - 3.0);
    return vec4(error, 0.0, 0.0, 1.0);
}

void main()
{
#if MODE <= 2
#if defined(USE_CORRECT)
    gl_FragColor = color_lit(n_correct);
#elif defined(USE_INVERT)
    gl_FragColor = color_lit(n_invert);
#elif defined(USE_COMBINE)
    gl_FragColor = color_lit(n_combine);
#endif
#else /* MODE > 2 */
#if defined(USE_INVERT)
    gl_FragColor = color_error(n_correct, n_invert);
#elif defined(USE_COMBINE)
    gl_FragColor = color_error(n_correct, n_combine);
#endif
#endif /* MODE > 2 */
}
