#version 110

vec4 iW[MAX_INFLUENCES];
ivec4 iB[MAX_GROUPS];
uniform mat4 uM[MAX_BONES];
uniform mat3 uiM[MAX_BONES];  /* inverse transpose matrices */

void initialise();
vec4 color_normalmap(vec3 normal);
vec4 color_lit(vec3 normal);
vec4 color_error(vec3 correct, vec3 normal);

vec3 transform_normal(const mat3 T)
{
    return normalize(cross(T[1], T[2]) * gl_Normal[0]
                     + cross(T[2], T[0]) * gl_Normal[1]
                     + cross(T[0], T[1]) * gl_Normal[2]);
}

`#'define `MODE' MODE
#if MODE == 0
#  define USE_CORRECT
#endif
#if MODE == 1
#  define USE_INVERT
#endif
#if MODE == 2
#  define USE_COMBINE
#endif
#if MODE == 3
#  define USE_CORRECT
#  define USE_INVERT
#endif
#if MODE == 4
#  define USE_CORRECT
#  define USE_COMBINE
#endif

#ifdef USE_CORRECT
varying vec3 n_correct;
#endif
#ifdef USE_INVERT
varying vec3 n_invert;
#endif
#ifdef USE_COMBINE
varying vec3 n_combine;
#endif
varying vec3 eyepos;

void main()
{
#ifdef USE_CORRECT
    mat3 N_correct = mat3(0.0);
#endif
#ifdef USE_INVERT
    mat3 N_invert = mat3(0.0);
#endif
#ifdef USE_COMBINE
    n_combine = vec3(0.0);
#endif

    initialise();
    vec4 v = vec4(0.0);
    for (int i = 0; i < MAX_INFLUENCES; i++)
    {
        int index = iB[i / 4][i % 4];
        vec4 vi = uM[index] * gl_Vertex;
        v += iW[i].w * vi;
#ifdef USE_CORRECT
        for (int j = 0; j < 3; j++)
            N_correct[j] += iW[i].w * vec3(uM[index][j]);
        for (int j = 0; j < 3; j++)
            N_correct[j] += iW[i][j] * vec3(vi);
#endif
#ifdef USE_INVERT
        for (int j = 0; j < 3; j++)
            N_invert[j] += iW[i].w * vec3(uM[index][j]);
#endif
#ifdef USE_COMBINE
        n_combine += iW[i].w * (uiM[index] * gl_Normal);
#endif
    }
#ifdef USE_CORRECT
    n_correct = gl_NormalMatrix * transform_normal(N_correct);
#endif
#ifdef USE_INVERT
    n_invert = gl_NormalMatrix * transform_normal(N_invert);
#endif
#ifdef USE_COMBINE
    n_combine = gl_NormalMatrix * n_combine;
#endif
    gl_Position = gl_ModelViewProjectionMatrix * v;
    eyepos = vec3(gl_ModelViewMatrix * v);
}
