/* Test code for the indexed priority queue class. This isn't in the
 * main build because the whole thing is implemented in a header file with
 * a C++ to naturally place the code.
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/extensions/HelperMacros.h>
#include "mesh/iheap.h"
#include <string>
#include <iostream>
#include <sstream>
#include <vector>
#include <utility>
#include <cstddef>

using namespace std;

class HeapTest : public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE(HeapTest);
    CPPUNIT_TEST(order);
    CPPUNIT_TEST(erase);
    CPPUNIT_TEST(modify);
    CPPUNIT_TEST_SUITE_END();

private:
    typedef indexed_priority_queue<string, int> PQ;
    typedef PQ::value_type value_type;
    vector<pair<string, int> > test_values;

    PQ *make_pq()
    {
        PQ *pq = new PQ;
        for (size_t i = 0; i < test_values.size(); i++)
            (*pq)[test_values[i].first] = test_values[i].second;
        return pq;
    }

    string itoa(int v)
    {
        ostringstream s;
        s << v;
        return s.str();
    }

public:
    void setUp()
    {
        test_values.resize(50);
        for (size_t i = 0; i < test_values.size(); i++)
        {
            int v = rand();
            test_values[i] = make_pair(itoa(v), v);
        }
    }

    void tearDown()
    {
    }

    void order()
    {
        PQ *pq = make_pq();
        int total = 0;
        int size = pq->size();
        int prev = INT_MAX;
        while (!pq->empty())
        {
            value_type cur = pq->top();
            pq->pop();
            CPPUNIT_ASSERT(itoa(cur.second) == cur.first);
            CPPUNIT_ASSERT(cur.second <= prev);
            prev = cur.second;
            total++;
        }
        CPPUNIT_ASSERT(total == size);
        delete pq;
    }

    void erase()
    {
        PQ *pq = make_pq();
        for (size_t i = 0; i < test_values.size(); i++)
            pq->erase(test_values[i].first);
        CPPUNIT_ASSERT(pq->empty());
        delete pq;
    }

    void modify()
    {
        PQ *pq = make_pq();
        for (size_t i = 0; i < test_values.size(); i++)
            (*pq)[test_values[i].first] = -test_values[i].second;
        CPPUNIT_ASSERT(pq->size() == test_values.size());
        while (!pq->empty())
        {
            value_type cur = pq->top();
            pq->pop();
            CPPUNIT_ASSERT(itoa(-cur.second) == cur.first);
        }
        delete pq;
    }
};

CPPUNIT_TEST_SUITE_REGISTRATION(HeapTest);
